
/*
 * Created on 26 fvr. 03
 *
 * To change this generated comment go to
 * Window>Preferences>Java>Code Generation>Code Template
 */
package graph.collectData;



import graph.weighted.Descriptions;
import graph.weighted.GraphData;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;


/**
 * @author Didier Croes
 * @version $Id: DataCollector.java,v 1.13 2004/03/24 16:08:47 didier Exp $
 *
 * this class collect Reaction by products and reactions by substratesfor the graphdataclass
 */
public class DataCollector {

  /**  */
  private Connection _connection;

  /** default jdbc connection string */
  private String _connectionString = "jdbc:oracle:thin:@oramamaze.ulb.ac.be:1521:amaze";

  /** default oracle jdbc driver */
  private String _dbDriverClass = "oracle.jdbc.driver.OracleDriver";

  /** default reaction by substrate query */
  private String _reactionbySubstrateQuery =
    "select dolcpd.AMAZE_ID COMPOUND_ID,dolreac.AMAZE_ID REACTION_ID from amaze.REACTION_LEVEL rl,amaze.DATABASE_OBJECT_LEVEL dolreac, amaze.DATABASE_OBJECT_LEVEL dolcpd,amaze.REACTION_SUBS rs where rl.OBJECT_ID = dolreac.OBJECT_ID and rs.REACT_ENTITY = dolcpd.OBJECT_ID and rl.OBJECT_ID = rs.OBJECT_ID order by COMPOUND_ID";

  /** default reaction by products query */
  private String _reactionbyProductsQuery =
    "select dolcpd.AMAZE_ID COMPOUND_ID,dolreac.AMAZE_ID REACTION_ID from amaze.REACTION_LEVEL rl,amaze.DATABASE_OBJECT_LEVEL dolreac, amaze.DATABASE_OBJECT_LEVEL dolcpd,amaze.REACTION_PRODS rs where rl.OBJECT_ID = dolreac.OBJECT_ID and rs.REACT_ENTITY = dolcpd.OBJECT_ID and rl.OBJECT_ID = rs.OBJECT_ID order by COMPOUND_ID";

    /** default name by amazeID quezy */
  private String _descriptionQuery = "select dol.amaze_id,don.original_name from amaze.database_object_level dol, amaze.database_object_names don, amaze.SP_COMPOUND_LEVEL scl where don.qualifier = 'primary' and don.object_id = dol.object_id and scl.object_id = dol.object_id";
  /**  */
  private Hashtable _reactionBySubstrates;

  /**  */
  private Hashtable _reactionByProducts;
  /**  */
  private Hashtable _nameByAmazeId;

  /**
   *
   * @param sUser the user name.
   * @param sPassword the password.
   */
  public void fillCompoundToReactionTable(String sUser, String sPassword) {
    this.fillAllTables (null, sUser, sPassword, null);
  }


  /**
   * fillCompoundToReactionTable.
   * @param dbDriverClass the DB driver class name.
   * @param sUser the user name.
   * @param sPassword the password.
   * @param connectionString String for the connection.
   */
  public void fillAllTables(String dbDriverClass, String sUser, String sPassword, String connectionString) {

    if (dbDriverClass != null) {
      this.setDbDriverClass (dbDriverClass);
    }

    if (connectionString != null) {
      this.setConnectionString (connectionString);
    }

    try {
      this.setConnection (DataCollector.connectToDB (this.getDbDriverClass (), sUser, sPassword,
                                                     this.getConnectionString ()));
      this.fillReactionByProducts ();
      this.fillReactionBySubstrates ();
      if (_descriptionQuery != null) this.fillNameByAmazeID(_descriptionQuery);
      else System.err.println("Warning: no description query!!");
      this.getConnection ().close ();
    } catch (Exception e) {
      System.out.println("Connecting :" + this.getConnectionString ())  ;
      e.printStackTrace ();
    } finally {
      try {
        this.getConnection ().close ();
      } catch (SQLException e) {
        e.printStackTrace ();
      }
    }
  }

  /**
   *      Static class to connect to database throught JDBC
   * @param dbDriverClass the DB driver class name.
   * @param sUser the user name.
   * @param sPassword the password.
   * @param connectionString String for the connection.
   * @return the connection.
   * @throws Exception if error occure.
   */
  public static Connection connectToDB (String dbDriverClass, String sUser, String sPassword, String connectionString)
    throws Exception {

    Connection connection = null;
    Driver dbDriver = (Driver) Class.forName (dbDriverClass).newInstance ();

    DriverManager.registerDriver (dbDriver);

    Properties props = new Properties ();

    if (sUser != null) {
      props.put ("user", sUser);
    }

    if (sPassword != null) {
      props.put ("password", sPassword);
    }

    props.put ("USE_METADATA", "true");
    DriverManager.setLoginTimeout (30);

    connection = DriverManager.getConnection (connectionString, props);

    connection.setTransactionIsolation (Connection.TRANSACTION_READ_COMMITTED);

    return connection;

    // dmd = connection.getMetaData ();
  }

  /**
   *      get a AMAZE_ID/Name table and fill the hashtable
   * @param query the query to be used to fill the name by amaze_id Hashtable.
   * @return the name by amazeID Hashtable.
   * @throws SQLException if there is an SQL exception.
   */
  
  protected Hashtable fillNameByAmazeID(String query) throws SQLException {
      Hashtable amazeIDTable = new Hashtable ();
      Statement stmt = this.getConnection ().createStatement ();
      ResultSet rs = stmt.executeQuery (query);
      ArrayList names = null;
      String currentID = null;//rs.getString(1),rs.getString(2)
      while (rs.next ()) {
        if(rs.getString(1) == null || !rs.getString(1).equals(currentID)){
            if (currentID != null) amazeIDTable.put(currentID,names);
            currentID = rs.getString(1);
            names = new ArrayList();
            
        }
          names.add(rs.getString(2));
      }
      rs.close();
      this._nameByAmazeId = amazeIDTable;
        return amazeIDTable;
  }
  
  /**
   *      get a compound/reactions table and fill the hashtable
   * @param query the query to be used to fill the reaction by compound Hashtable.
   * @return the reaction by compound Hashtable.
   * @throws SQLException if there is an SQL exception.
   */
  protected Hashtable fillReactionByCpd (String query)
    throws SQLException {

    Hashtable cpdTable = new Hashtable ();
    Statement stmt = this.getConnection ().createStatement ();
    ResultSet rs = stmt.executeQuery (query);
    System.out.println ("Query =" + query);
    String currentCpd = null;
    ArrayList reactions = null;
    int i = 0;

    while (rs.next ()) {
      if ((currentCpd == null) || !currentCpd.equals (rs.getString (1))) {
        if (currentCpd != null) {
          cpdTable.put (currentCpd, reactions);
          i++;
        }

        reactions = new ArrayList ();
        currentCpd = rs.getString (1);
      }

      reactions.add (rs.getString (2));
    }

    // LAST REACTION
    if (reactions != null) {
      cpdTable.put (currentCpd, reactions);
    }
    rs.close();
    System.out.println("Compounds : " +i);
    return cpdTable;
  }

  /**
   *    fill the reaction(value) by substrates(key) hashtable
   *    @throws SQLException if there is a sql exception
   */
  protected void fillReactionBySubstrates ()
    throws SQLException {
    this.setReactionBySubstrate (this.fillReactionByCpd (this.getReactionbySubstrateQuery ()));
  }

  /**
   * Fill the reaction(value) by products(key) hashtable
   * @throws SQLException an sql exception.
   */
  protected void fillReactionByProducts ()
    throws SQLException {
    this.setReactionByProduct (this.fillReactionByCpd (this.getReactionbyProductsQuery ()));
  }

  /**
   * The main method.
   * @param args the arguments
   */
  public static void main (String [] args) {

    System.out.println ("That's the beginning.");
    String spath = System.getProperty("user.home" ); 
    DataCollector app = new DataCollector ();
    //app.fillCompoundToReactionTable("amaze","amaze");
    GraphData graphData = app.buildGraphData(null,"amaze","amaze","jdbc:oracle:thin:@amaze.ulb.ac.be:1521:master",null,null);

    graphData.store (spath +"/graphData.gdt");
    Descriptions desc = app.getDescriptions();
    desc.store(spath + "/graphData.desc");
    System.out.println ("That's the end.");
  }

  /**
   * @return Connection
   */
  public Connection getConnection () {
    return _connection;
  }

  /**
   * @return String
   */
  public String getConnectionString () {
    return _connectionString;
  }

  /**
   * @return Hashtable
   */
  public Hashtable getReactionByProducts () {
    return _reactionByProducts;
  }

  /**
   * @return Hashtable
   */
  public Hashtable getReactionBySubstrate () {
    return _reactionBySubstrates;
  }

  /**
   * Sets the connection.
   * @param connection The connection to set
   */
  public void setConnection (Connection connection) {
    this._connection = connection;
  }

  /**
   * Sets the connectionString.
   * @param connectionString The connectionString to set
   */
  public void setConnectionString (String connectionString) {
    this._connectionString = connectionString;
  }

  /**
   * Sets the reactionByProducts.
   * @param reactionByProduct set the reaction by product Hashtable.
   */
  public void setReactionByProduct (Hashtable reactionByProduct) {
    this._reactionByProducts = reactionByProduct;
  }

  /**
   * Sets the reactionBySubstrates.
   * @param reactionBySubstrate set the reaction by substrate Hashtable.
   */
  public void setReactionBySubstrate (Hashtable reactionBySubstrate) {
    this._reactionBySubstrates = reactionBySubstrate;
  }

  /**
   * @return String
   */
  public String getDbDriverClass () {
    return _dbDriverClass;
  }

  /**
   * Sets the dbDriverClass.
   * @param dbDriverClass The dbDriverClass to set
   */
  public void setDbDriverClass (String dbDriverClass) {
    this._dbDriverClass = dbDriverClass;
  }

  /**
   * @return String
   */
  public String getReactionbyProductsQuery () {
    return _reactionbyProductsQuery;
  }

  /**
   * @return String
   */
  public String getReactionbySubstrateQuery () {
    return _reactionbySubstrateQuery;
  }

  /**
   * Sets the reactionbyProductsQuery.
   * @param reactionbyProductsQuery The reactionbyProductsQuery to set
   */
  public void setReactionbyProductsQuery (String reactionbyProductsQuery) {
    this._reactionbyProductsQuery = reactionbyProductsQuery;
  }

  /**
   * Sets the reactionbySubstrateQuery.
   * @param reactionbySubstrateQuery The reactionbySubstrateQuery to set
   */
  public void setReactionbySubstrateQuery (String reactionbySubstrateQuery) {
    this._reactionbySubstrateQuery = reactionbySubstrateQuery;
  }
/**
 * get name by amaze_id table
 * @return name by amazeID table
 */
public Hashtable getNameByAmazeId() {
    return _nameByAmazeId;
}

/**
 * buildGraphData
 * @param dbDriverClass the DB driver class name.
 * @param sUser the user name.
 * @param sPassword the password.
 * @param connectionString String for the connection.
 * @param reactionBySubstratesQuery the query to get the reaction by subtrate resultset
 * @param reactionByProductsQuery the query to get the reaction by product resultset
 */
public GraphData buildGraphData (String dbDriverClass, String sUser, String sPassword, String connectionString,String reactionBySubstratesQuery,String reactionByProductsQuery) {
  GraphData gd = new GraphData ();
  if (reactionBySubstratesQuery != null) this.setReactionbySubstrateQuery(reactionBySubstratesQuery);
  if (reactionByProductsQuery != null) this.setReactionbyProductsQuery(reactionByProductsQuery);
  this.fillAllTables (dbDriverClass, sUser, sPassword, connectionString);
  gd.setReactionByProduct (this.getReactionByProducts ());
  gd.setReactionBySubstrate (this.getReactionBySubstrate ());
  gd.invertTables ();
  return gd;
}

public Descriptions getDescriptions(){
    if(this.getNameByAmazeId() != null) return new Descriptions(this.getNameByAmazeId());
    else return null;
}

	/**
	 * @return
	 */
	public String getDescriptionQuery() {
		return _descriptionQuery;
	}

	/**
	 * @param string
	 */
	public void setDescriptionQuery(String string) {
		_descriptionQuery = string;
	}

}


/*--- Formatted in North Style ---*/
