/*******************************************************************************
 * Copyright (c) 2003 North Bears Team
 * All rights reserved.
 *
 * North Bears Team: Erick Antezana, Fabian Couche, Didier Croes, 
 * Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * 
 *******************************************************************************/

package graph.weighted;



import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;


/**
 * Class: Descriptions Description: buffer between the biological data and the
 * graph structure.
 *
 * @author Fabian Couche
 * @version $Revision: 1.6 $
 */
public class Descriptions
  implements Serializable {

  /** Field amazeIDByName */
  private Hashtable _amazeIDByName;

  /** Field nameByAmazeID */
  private Hashtable _nameByAmazeID;

  /**
   * Descriptions Constructor
   */
  public Descriptions () {
    _amazeIDByName = new Hashtable ();
    _nameByAmazeID = new Hashtable ();
  }

  /**
   * Reads a saved GraphData using the parameterDescriptions Constructor
   *
   * @param fileName
   *            A String
   */
  public Descriptions (String fileName) {
    this (fileName, false);
  }

  /**
   * Reads a saved GraphData using the parameterDescriptions Constructor
   * @param fileName
   * @param ascii
  */
  public Descriptions (String fileName, boolean ascii) {

    if (ascii) {
      System.out.println ("before read ascii");
      this.readAsciiDescriptions (fileName);
      System.out.println ("after read ascii");
    } else {
      Descriptions temp = read (fileName);

      this._nameByAmazeID = temp._nameByAmazeID;
    }

    this._amazeIDByName = GraphData.invert (_nameByAmazeID);
  }

  /**
   * build description from name by amazeID map
   *
   * @param nameByAmazeID
   *            name by amazeID map
   */
  public Descriptions (Map nameByAmazeID) {
    this.setNameByAmazeID (nameByAmazeID);
  }

  /**
   * Inverts the
   *
   */

  /*
   * private Hashtable invert (Hashtable inTable) {
   *
   * Hashtable outTable = new Hashtable (); Object key, x; Enumeration input =
   * inTable.keys ();
   *
   * while (input.hasMoreElements ()) { key = input.nextElement (); x =
   * inTable.get (key);
   *
   * outTable.put (x, key); }
   *
   * return (outTable); }
   */

  /**
   * Is necessary to be able to serialize the data.
   *
   * @param out
   *            the produced stream.
   */
  protected void writeObject (ObjectOutputStream out) {

    try {
      out.writeObject (_nameByAmazeID);
    } catch (Exception e) {
      System.out.println ("Exception during serialization: " + e);
      System.exit (0);
    }
  }

  /**
   * Is necessary to be able to serialize the data.
   *
   * @param in
   *            A given stream.
   * @throws ClassNotFoundException
   *             a exception during reading.
   * @throws IOException
   *             a exception during input-output.
   */
  protected void readObject (java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {

    _nameByAmazeID = (Hashtable) in.readObject ();

    System.out.println ("nameByAmazeID read\t\t" + _nameByAmazeID.size ());
  }

  /**
   * Sets the table of Name by String
   *
   * @param rp
   *            A Map
   */
  public void setNameByAmazeID (Map rp) {
    _nameByAmazeID = new Hashtable (rp);
    this._amazeIDByName = GraphData.invert (_nameByAmazeID);
  }

  /**
   * Gives the name corresponding to the identifier.
   *
   * @param ident
   *            the identifier.
   * @return the name.
   */
  public String getName (String ident) {

    String n = (String) getNames (ident).get (0);

    if (n == null) {
      n = ident;
    }

    return n;
  }

  /**
   *
   * @param ident
   * @return
   */
  public ArrayList getNames (String ident) {
    return (ArrayList) _nameByAmazeID.get (ident);
  }

  /**
   *
   * @param name
   * @return
   */
  public ArrayList getIDs (String name) {
    return (ArrayList) _amazeIDByName.get (name);
  }

  /**
   * Gives the ObjectID from a name.
   *
   * @param n
   *            a name.
   * @return the identifier.
   * @throws Exception
   *             a exception for a unknown name.
   */
  public String getAmazeID (String n)
    throws Exception {

    if (!_amazeIDByName.containsKey (n)) {
      throw new Exception ("Unknown name '" + n + "'");
    }

    String ident = (String) ((ArrayList) _amazeIDByName.get (n)).get (0);

    return ident;
  }

  /**
   *
   * @param name
   * @return
   */
  public Hashtable getIdsByPartialName (String name) {

    Hashtable ret = new Hashtable ();

    if (name.endsWith ("-")) {
      name = name.substring (0, name.length () - 1);
    }

    Iterator itr = this._amazeIDByName.keySet ().iterator ();

    while (itr.hasNext ()) {
      String kc = (String) itr.next ();

      if ((kc.indexOf (name) > -1) && !kc.endsWith ("-")) {
        ret.put (kc, this._amazeIDByName.get (kc));
      }
    }

    return ret;
  }

  /**
   * Saves a GraphData in a file.
   *
   * @param fileName
   *            the name of the file to save in.
   */
  public void store (String fileName) {

    Descriptions descript = this;

    try {
      FileOutputStream fos = new FileOutputStream (fileName);
      ObjectOutputStream oos = new ObjectOutputStream (fos);

      oos.writeObject (descript);
      oos.flush ();
      oos.close ();
    } catch (Exception e) {
      System.out.println ("Exception during serialization: " + e);
      System.exit (0);
    }
  }

  /**
   * Reads a GraphData in a file.
   *
   * @param fileName
   *            A String
   * @return the Descriptions
   */
  public Descriptions read (String fileName) {

    Descriptions back = new Descriptions ();

    try {
      FileInputStream fis = new FileInputStream (fileName);
      ObjectInputStream ois = new ObjectInputStream (fis);

      back = (Descriptions) ois.readObject ();

      ois.close ();
    } catch (Exception e) {
      System.out.println ("Exception during deserialization: " + e);
      System.exit (0);
    }

    return (back);
  }

  /**
   *
   * @param filename
   */
  public void saveAsciiDescriptions (String filename) {

    try {
      BufferedWriter bw = new BufferedWriter (new FileWriter (filename));
      Iterator itr = this._nameByAmazeID.entrySet ().iterator ();

      while (itr.hasNext ()) {
        Map.Entry entry = (Map.Entry) itr.next ();
        String in = (String) entry.getKey ();
        Iterator nameItr = ((ArrayList) entry.getValue ()).iterator ();

        while (nameItr.hasNext ()) {
          bw.write (in + "\t" + nameItr.next () + "\n");
        }
      }

      bw.flush ();
      bw.close ();
    } catch (IOException e) {
      System.out.println ("Exception during writing ascii file: " + e);
      System.exit (-1);
    }
  }

  /**
   *
   * @param filename
   */
  public void readAsciiDescriptions (String filename) {

    try {
      BufferedReader br = new BufferedReader (new FileReader (filename));
      String s = br.readLine ();

      if (s != null) {
        Hashtable nameByAmazeID = new Hashtable ();
        String id = null;
        ArrayList names = new ArrayList ();

        while ((s = br.readLine ()) != null) {

          // System.out.println(s);
          StringTokenizer str = new StringTokenizer (s, "\t");
          String temp = str.nextToken ();

          if ((id != null) && !id.equals (temp)) {
            if (nameByAmazeID.containsKey (id)) {
              ((ArrayList) nameByAmazeID.get (id)).addAll (names);
            } else {
              nameByAmazeID.put (id, names);
            }

            names = new ArrayList ();
            id = temp;

            String name = str.nextToken ();

            if (name != null) {
              names.add (name);
            }
          } else {
            if (id == null) {
              names = new ArrayList ();
              id = temp;
            }

            String name = str.nextToken ();

            if (name != null) {
              names.add (name);
            }
          }
        }

        if (nameByAmazeID.containsKey (id)) {
          ((ArrayList) nameByAmazeID.get (id)).addAll (names);
        } else {
          nameByAmazeID.put (id, names);
        }

        this.setNameByAmazeID (nameByAmazeID);
      } else {
        throw new IOException ("File is empty");
      }
    } catch (IOException e) {
      System.out.println ("Exception during reading ascii file: " + e);
      System.exit (-1);
    }
  }
}


/*--- Formatted in North Style ---*/
