package graph.weighted;



import gnu.getopt.Getopt;
import graph.weighted.weightedNodes.Graph;
import graph.weighted.weightedNodes.Solutions;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;


/**
 * Class: PathfindingApp
 * Description: Application to test pathfinding on weighted graph
 *
 * @author Fabian Couche
 * @version $Revision: 1.3 $
 */
public class PathfindingApp {

  /** Field sourceName */
  private static String _sourceName;

  /** Field targetName */
  private static String _targetName;

  /** Field sourceID */
  private static String _sourceID;

  /** Field targetID */
  private static String _targetID;

  /** Field nbSol */
  private static int _nbSol = 0;

  /** Field gData */
  private static GraphData _gData;

  /** Field descript */
  private static Descriptions _descript;

  /** Field g */
  private static Graph _graph;

  /**
   * PathfindingApp Constructor
   */
  public PathfindingApp () {
  }

  /**
   * setNbPathways Method
   * @param nb the number of paths.
   */
  public static void setNbPathways (int nb) {
    _nbSol = nb;
  }

  /**
   * setSourceCompoundName Method
   * @param s the source.
   * @throws Exception an exception.
   */
  public static void setSourceCompoundName (String s)
    throws Exception {
    _sourceName = s;
    _sourceID = _descript.getAmazeID (s);
  }

  /**
   * setTargetCompoundName Method
   * @param s the target.
   * @throws Exception an exception.
   */
  public static void setTargetCompoundName (String s)
    throws Exception {
    _targetName = s;
    _targetID = _descript.getAmazeID (s);
  }

  /**
   * reloadGraphData Method
   * @param file the name of the file.
   */
  public static void reloadGraphData (String file) {

    // 
    assert file != null;

    // 
    _gData = new GraphData (file);
  }

  /**
   * reloadDescriptions Method
   * @param file the name of the file.
   */
  public static void reloadDescriptions (String file) {

    // 
    assert file != null;

    // 
    _descript = new Descriptions (file);
  }

  /**
   * lire Method
   * @return A String
   */
  public static String lire () {

    String line = "0";
    BufferedReader entree = new BufferedReader (new InputStreamReader (System.in));

    try {
      line = entree.readLine ();
    } catch (Exception e) {

      // do nothing.
    }

    return (line);
  }

  /**
   * main Method
   * @param args the arguments.
   * @throws Exception an exception.
   */
  public static void main (String [] args)
    throws Exception {

    Solutions sol = new Solutions ();
    DateFormat infoTime = new SimpleDateFormat ("HH.mm.ss");
    String arg, graphDataFile, descriptionsFile, source, target, outputFile;
    float weightMax, w;
    int c, nbPath, nbReact, algo;
    boolean output = false;

    nbPath = 10;
    nbReact = 5;
    algo = 0;
    weightMax = 500;
    w = 0;
    arg = new String ();
    graphDataFile = new String ("metabolic.gdt");
    descriptionsFile = new String ("metabolic.dsc");
    source = new String ();
    target = new String ();
    outputFile = null;

    Getopt gt = new Getopt ("pathfinding", args, "g:d:n:m:w:r:a:s:t:o");

    while ((c = gt.getopt ()) != -1) {
      switch (c) {

        case 'g' :
          graphDataFile = gt.getOptarg ();
          break;

        case 'd' :
          descriptionsFile = gt.getOptarg ();
          break;

        case 'n' :
          arg = gt.getOptarg ();
          nbPath = Integer.parseInt (arg);
          break;

        case 'm' :
          arg = gt.getOptarg ();

          if (arg.equals ("const")) {
            w = 1;
          }
          break;

        case 'w' :
          arg = gt.getOptarg ();
          weightMax = Float.parseFloat (arg);
          break;

        case 'r' :
          arg = gt.getOptarg ();
          nbReact = Integer.parseInt (arg);
          break;

        case 'a' :
          arg = gt.getOptarg ();
          algo = Integer.parseInt (arg);
          break;

        case 's' :
          source = gt.getOptarg ();
          break;

        case 't' :
          target = gt.getOptarg ();
          break;

        case 'o' :
          output = true;
          break;

        case '?' :
          break;  // getopt() already printed an error

        default :
          System.out.print ("getopt() returned " + c + "\n");
      }
    }

    if ((algo < 0) || (algo > 2)) {
      System.out.println ("Wrong number of algorithm!");
    } else {
      if (output == true) {
        outputFile = new String (source + target + ".sol");
      }

      System.out.println ();
      System.out.println ("Welcome in the test application");
      System.out.println ();
      System.out.println (infoTime.format (new Date ()) + " Collecting GraphData...");
      reloadGraphData (graphDataFile);
      System.out.println (infoTime.format (new Date ()) + " Collecting Descriptions.");
      reloadDescriptions (descriptionsFile);
      System.out.println (infoTime.format (new Date ()) + " Data collected!");
      System.out.println ();
      System.out.println (infoTime.format (new Date ()) + " Constructing graph...");

      if (w == 0) {
        _graph = _gData.getGraph ();
      } else {
        _graph = _gData.getGraph (w);
      }

      // gData=new GraphData();
      // g=gData.exemple();
      System.out.println (infoTime.format (new Date ()) + " Graph constructed!");
      System.out.println ();

      try {
        setSourceCompoundName (source);
      } catch (Exception e) {
        System.out.println (e.getMessage ());
      }

      try {
        setTargetCompoundName (target);
      } catch (Exception e) {
        System.out.println (e.getMessage ());
      }

      // sourceID=source;
      // targetID= target;
      setNbPathways (nbPath);

      if (_nbSol != 0) {
        System.out.println (infoTime.format (new Date ()) + " Pathfinding started!!!");
        sol.setPathways ((ArrayList) _graph.pathFinding (_nbSol, _sourceID, _targetID, weightMax, nbReact, algo));
        System.out.println (infoTime.format (new Date ()) + " Pathfinding ended!!!");
        System.out.println ();

        if (sol.isEmpty ()) {
          System.out.println ("Timeout!!!");
        } else {
          if (outputFile != null) {
            System.out.println ("Record of the solution.");
            sol.store (outputFile);
            System.out.println ("Solution recorded!");
          }

          sol.textDisplay (_descript);

          // sol.textDisplay();
          // sol.graphicDisplay(descript);
          System.out.println ("Export diagram to file.");
          System.out.println ("Diagram exported!");
        }
      }

      // ArrayList test=sol.getPathways();
      System.out.println ();
      System.out.println ("Bye!");
      System.out.println ();
    }
  }
}


/*--- Formatted in North Style ---*/
