package graph.weighted.weightedNodes;

import graph.weighted.GraphData;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;

/**
 * Backtracking
 * 
 * Warning: 
 * 1) Karoline Faust changed line 403 from 
 * backtrack(futurNode);
 * to
 * backtrackRanked(futurNode);
 * 2) Karoline Faust added two lines (393 and 394) and
 * outcommented line 392
 * 
 * @author Fabian Couche
 * @version $Id: Backtracking.java,v 1.13 2005/02/25 10:12:19 didier Exp $
 */
public class Backtracking implements Runnable,PathFindingInterface {
	// //////////////////////////////////////////////////////////
	// 
	// Access
	// 
	// //////////////////////////////////////////////////////////

	/**
	 * Sets the graph to use.
	 * 
	 * @param graph
	 *            the current graph.
	 */
	protected void setGraph(Graph graph) {
		_graph = graph;
	}

	/**
	 * Returns the used graph.
	 * 
	 * @return the current graph.
	 */
	protected Graph getGraph() {
		return _graph;
	}

	/**
	 * Sets the id of the start point.
	 * 
	 * @param startPoint
	 *            the id of the start point.
	 */
	protected void setStartPoint(String startPoint) {
		_startPointId = startPoint;
	}

	/**
	 * Returns the id of the start point.
	 * 
	 * @return the id of the start point.
	 */
	protected String getStartPoint() {
		return _startPointId;
	}

	/**
	 * Sets the id of the end point.
	 * 
	 * @param endPoint
	 *            the id of the end point.
	 */
	protected void setEndPoint(String endPoint) {
		_endPointId = endPoint;
	}

	/**
	 * Returns the id of the end point.
	 * 
	 * @return the id of the end point.
	 */
	protected String getEndPoint() {
		return _endPointId;
	}

	
	

	// //////////////////////////////////////////////////////////
	// 
	// Basic operations
	// 
	// //////////////////////////////////////////////////////////

	/**
	 * Is Called when the backtracking is launched as a thread.
	 */
	public void run() {
		
		if (!this._isInitialized) {
			System.err.println("Backtracking was not initialized!");
			System.exit(-1);

			// throw new Exception ("Backtracking was not initialized!");
		} else {
			if(backtracking2 != null){
				if (!backtracking2._isInitialized) {
					System.err.println("Backtracking2 was not initialized!");
					System.exit(-1);
				}else{
					backtrackThread2 = new Thread(backtracking2, "Backtracking2");
					System.out.println("Starting thread 2");
					backtrackThread2.start();
					
				}
				
			}
			String identifier = startNode.getId();
			this._currentWay.add(0, identifier);

			this._currentDistance = startNode.getWeight();
			System.out.println("Starting thread 1");
			if (this._byRank)
				try {
					this.backtrackRanked(startNode);
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			else
				try {
					this.backtrack(startNode);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		}
		if (backtrackThread2 !=null){
			try {
				backtrackThread2.join();
				System.out.println("backtrackThread2 is joined");
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				System.out.println("backtrackThread2 is interrupted");
			}
		}
		if (backtrackThread2 !=null || this.isCompound())	{
			if (_byRank) {

				this.setSolutionsMap(tempSolutions.getTempSolutionsMap());
				ArrayList templist  = new ArrayList();
				Iterator itr = this.getSolutionsMap().values().iterator();
				while (itr.hasNext()) {
					templist.addAll((ArrayList) itr.next());
				}
				_solutions = new Solutions();
				_solutions.addPathways(templist);

			} else {
				this.setSolutions(tempSolutions.getSolutions());
				this.setSolutionsMap(null);
			}
			// the array is empty
			tempSolutions.getCallingPathfinding().notifyBacktrackEnd(); // job is finnished
	
		}
				
		// by
		// backtracking

		// return _solutions;
	}

	/**
	 * Initializes all what is needed by the backtracking.
	 * 
	 * @param graph
	 *            the used graph.
	 * @param startPointId
	 *            the start point.
	 * @param endPointId
	 *            the end point.
	 * @param nbSolutions
	 *            the number of solutions to obtain.
	 * @param isByRank
	 *            if the number of solution is by ranking
	 * @param maxDistance
	 *            the maximum distance.
	 * @param maxLevel
	 *            the maximum level.
	 * @param timeout
	 *            the backtraking level.
	 * @param callingPathfinding
	 *            calling class for starting the REA in backtrack_rea mode.
	 * @param onlyBacktrack
	 *            onlybacktrak or backtrack followed by rea mode.
	 * @throws Exception
	 *  
	 */
	public void init(GraphData graphdata, String startPointId, String endPointId,int mode,
			int nbSolutions, boolean isByRank, float maxDistance, int maxLevel,
			long timeout, Pathfinding callingPathfinding, boolean onlyBacktrack) throws Exception {
		tempSolutions = new BactrackSolutions();
		this.setMode(mode);
		this.setGraphData(graphdata);
		this.setGraph(this.getNewGraph(this.getGraphData(),mode));
		this.setStartPoint(startPointId);
		this.setEndPoint(endPointId);
		tempSolutions.setNbSolutions(nbSolutions);
		tempSolutions.setMaxDistance(maxDistance);
		tempSolutions.setMaxLevel(maxLevel);
		tempSolutions.setTimeout(timeout);
		tempSolutions.setCallingPathfinding(callingPathfinding);
		tempSolutions.setOnlyBacktrack(onlyBacktrack);
		this.setByRank(isByRank);
		_currentWay = new LinkedList();
		_currentLevel = 1;
		this._isInitialized = true;
		prepareBacktracking();
	}
	public void init(Graph graph, Node startNode, String endPointId,int mode, boolean isByRank,BactrackSolutions tempSolutions) throws Exception{
		//this.setGraphData(graphdata);
		this.setMode(mode);
		this.setGraph(graph);
		this.tempSolutions = tempSolutions;
		//this.setStartPoint(startPointId);
		this.setStartNode(startNode);
		this.setEndPoint(endPointId);
		this.setByRank(isByRank);
		_currentWay = new LinkedList();
		_currentLevel = 1;
		this._isInitialized = true;
	}
	
	public void prepareBacktracking() throws Exception{
		tempSolutions.setStarttime(System.currentTimeMillis());

		this._currentLevel = 1;
		this.setCompound(this.getGraph().containsCompoundId(this.getStartPoint()));
		if (!this.isCompound()) {
			LinkedList couple = this.getGraph().getReactionPositionsById(
					this.getStartPoint());
			
			Integer startPos1 = (Integer) couple.getFirst();
			Integer startPos2 = (Integer) couple.getLast();

			startNode = (Node) this.getGraph().getNodeByPos(
					startPos1.intValue());
			backtracking2 = new Backtracking();
			Graph g = this.getNewGraph(this.getGraphData(),this.getMode());
			LinkedList couple2 = g.getReactionPositionsById(
					this.getStartPoint());
			
			Integer startPos21 = (Integer) couple2.getFirst();
			Integer startPos22 = (Integer) couple2.getLast();
			Node startNode2=(Node) g.getNodeByPos(
					startPos21.intValue());
			if (startNode.isDirect() == startNode2.isDirect()){
				startNode2=(Node) g.getNodeByPos(
						startPos22.intValue());
			}
			backtracking2.init(g,startNode2,this.getEndPoint(),this.getMode(),this.isByRank(),this.tempSolutions);
			
		} else {
			int startPos = this.getGraph().getCompoundPositionById(this.getStartPoint());
			startNode = (Node) this.getGraph().getNodeByPos(startPos);
		}
	}
	
	/**
	 * Does a pathfinding using an optimised recursive algorithm of
	 * backtracking.
	 * 
	 * @param currentNode
	 *            the current node.
	 * @throws Exception
	 */
	private void backtrack(Node currentNode) throws Exception {

		// 
		assert currentNode != null;

		// 
		Node futurNode; // next node
		Integer futur; // position of next node

		if (currentNode.getId().equals(this._endPointId)) {
			//updateSolution(this._byRank);
			tempSolutions.updateSolutionKShortest(this._currentWay,this._currentDistance);
		} else {
			if (this._currentLevel < tempSolutions.getMaxLevel()) {
				for (Iterator i = currentNode.getNxt().iterator(); i
						.hasNext();) {
					futur = (Integer) i.next();
					futurNode = (Node) this.getGraph().getNodeByPos(
							futur.intValue());

					if (!futurNode.marked()) {
						this._currentDistance += futurNode.getWeight();

						if (this._currentDistance < tempSolutions.getMaxDistance()) {
							++_currentLevel;

							currentNode.mark();
							this._currentWay.addLast(futurNode.getId());
							backtrack(futurNode);
							this._currentWay.removeLast();
							currentNode.unmark();

							--this._currentLevel;
						}

						this._currentDistance -= futurNode.getWeight();
					}
				}
			}
		}
	}

	private void backtrackold(Node currentNode) throws Exception {

		// 
		assert currentNode != null;

		// 
		Node futurNode; // next node
		Integer futur; // position of next node
		
		if (currentNode.getId().equals(this._endPointId)) {
			//updateSolution(this._byRank);
			tempSolutions.updateSolutionKShortest(this._currentWay,this._currentDistance);
		} else {
			if (this._currentLevel < tempSolutions.getMaxLevel()) {
				for (Iterator i = currentNode.getNxt().iterator(); i
						.hasNext();) {
					//futur = (Integer) i.next();
					//System.out.println("currentnode:"+currentNode.getId() +"\tisdirect:"+currentNode.isDirect());
					//System.out.println(i.next().getClass().getName());
					futur = (Integer) i.next();
					futurNode = (Node) this.getGraph().getNodeByPos(
							futur.intValue());
					//System.out.println("futurNode:"+currentNode.getId() +"\tisdirect:"+currentNode.isDirect());
					if (!futurNode.marked()) {
						this._currentDistance += futurNode.getWeight();
						
						if (this._currentDistance < tempSolutions.getMaxDistance()) {
							++_currentLevel;

							currentNode.mark();
							this._currentWay.addLast(futurNode.getId());
							backtrack(futurNode);
							this._currentWay.removeLast();
							currentNode.unmark();

							--this._currentLevel;
						}

						this._currentDistance -= futurNode.getWeight();
					}
				}
			}
		}
	}

	/**
	 * Does a pathfinding using an optimised recursive algorithm of
	 * backtracking.
	 * 
	 * @param currentNode
	 *            the current node.
	 * @throws Exception
	 */
	private void backtrackRanked(Node currentNode) throws Exception {

		// 
		assert currentNode != null;

		// 
		Node futurNode; // next node
		Integer futur; // position of next node

		if (currentNode.getId().equals(this._endPointId)) {
			//updateSolution(this._byRank);
			tempSolutions.updateSolutionKRank(_currentWay,_currentDistance);
		} else {
			if (this._currentLevel < tempSolutions.getMaxLevel()) {
				for (Iterator i = currentNode.getNxt().iterator(); i
						.hasNext();) {
					// futur = (Integer) i.next();
					//futurNode = (Node) i.next();
					// next two lines added by Karoline Faust:
					futur = (Integer) i.next();
					futurNode = (Node) this.getGraph().getNodeByPos(futur.intValue());
					if (!futurNode.marked()) {
						this._currentDistance += futurNode.getWeight();

						if (this._currentDistance < tempSolutions.getMaxDistance()) {
							++_currentLevel;

							currentNode.mark();
							this._currentWay.addLast(futurNode.getId());
							backtrackRanked(futurNode);
							this._currentWay.removeLast();
							currentNode.unmark();

							--this._currentLevel;
						}

						this._currentDistance -= futurNode.getWeight();
					}
				}
			}
		}
	}

	/**
	 * @return
	 */
	public Solutions getSolutions() {
		return _solutions;
	}

	/**
	 * @param list
	 */
	private void setSolutions(Solutions sol) {
		_solutions = sol;
	}

	/**
	 * @return
	 */
	
	

	// //////////////////////////////////////////////////////////
	// 
	// Implementation
	// 
	// //////////////////////////////////////////////////////////

	/**  */
	private Graph _graph;
	private GraphData _graphData;
    protected Backtracking backtracking2 = null;
	/**  */
	private String _startPointId;

	/**  */
	private String _endPointId;

	/**  */
	private int _currentLevel;
  protected BactrackSolutions tempSolutions;

	/**  */
	protected int mode;
	/**  */
	private float _currentDistance;

	/**  */
	private LinkedList _currentWay;
    protected boolean compound= false;

	/**  */
	private Solutions _solutions;

	private TreeMap _solutionsMap;

	/**  */
	private boolean _isInitialized = false;

	Node startNode;
	

	private boolean _byRank = false;

	
	Thread backtrackThread2 = null;
	/**
	 * @return Returns the _byRank.
	 */
	public boolean isByRank() {
		return _byRank;
	}

	/**
	 * @param rank
	 *            The _byRank to set.
	 */
	public void setByRank(boolean byrank) {
		_byRank = byrank;
	}

	/**
	 * @return Returns the _solutionsMap.
	 */
	public TreeMap getSolutionsMap() {
		return _solutionsMap;
	}

	/**
	 * @param map
	 *            The _solutionsMap to set.
	 */
	public void setSolutionsMap(TreeMap map) {
		_solutionsMap = map;
	}
	/**
	 * @return Returns the _graphData.
	 */
	public GraphData getGraphData() {
		return _graphData;
	}
	/**
	 * @param data The _graphData to set.
	 */
	public void setGraphData(GraphData data) {
		_graphData = data;
	}
	public Graph getNewGraph(GraphData graphdata, int PFMethod ) throws Exception{
		switch (PFMethod) {// get the graph requested
		case WHEIGTHED_MODE :
			return graphdata.getGraph();
			
		case EXCLUSION_MODE :
			return graphdata.getGraph(1);
		
		case RAW_MODE :
			return graphdata.getGraph(1);	
		case MODIFIED_WHEIGTHED_MODE :
			return graphdata.getModifiedGraph();	
		default :
			throw new Exception("Not a valid Mode!");
		
	// break;
	}
	}
	/**
	 * @return Returns the mode.
	 */
	public int getMode() {
		return mode;
	}
	/**
	 * @param mode The mode to set.
	 */
	public void setMode(int mode) {
	}
	
	public boolean areEqual(Node node1, Node node2){
		if (node1.getId().equals(node2.getId())) {
			return true;
		}
		else return false;
	}
	/**
	 * @return Returns the startNode.
	 */
	public Node getStartNode() {
		return startNode;
	}
	/**
	 * @param startNode The startNode to set.
	 */
	public void setStartNode(Node startNode) {
		this.startNode = startNode;
	}
	/**
	 * @return Returns the compound.
	 */
	public boolean isCompound() {
		return compound;
	}
	/**
	 * @param compound The compound to set.
	 */
	public void setCompound(boolean compound) {
		this.compound = compound;
	}
}

/*--- Formatted in North Style ---*/
