package graph.weighted.weightedNodes;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

public class BactrackSolutions {
	protected int _nbSolutions;
	/**  */
	protected int _maxLevel;
	/**  */
	protected float _maxDistance;
	/**  */
	protected long _timeout = -1;

	/**  */
	protected long _starttime;

	/**  */
	protected Pathfinding callingPathfinding;
	protected boolean onlyBacktrack = false;
	
	/**  */
	protected ArrayList _tempsolutions;

	protected TreeMap _tempsolutionsmap;


	public BactrackSolutions() {
		super();
		_tempsolutions = new ArrayList();
		_tempsolutionsmap = new TreeMap();

		// TODO Auto-generated constructor stub
	}
	/*
	 * public void init (Graph graph, String startPointId, String endPointId,
	 * int nbSolutions, float maxDistance, int maxLevel, Pathfinding
	 * callingPathfinding, long timeout,boolean onlyBacktrack) {
	 * this.init(graph, startPointId, endPointId,nbSolutions,false,
	 * maxDistance,maxLevel,timeout,callingPathfinding,onlyBacktrack);
	 *  } public void init (Graph graph, String startPointId, String endPointId,
	 * int nbSolutions, float maxDistance, int maxLevel, Pathfinding
	 * callingPathfinding, long timeout) { this.init(graph, startPointId,
	 * endPointId,nbSolutions,false,
	 * maxDistance,maxLevel,timeout,callingPathfinding,false);
	 *  }
	 */
	protected synchronized void updateSolution(List currentWay, float currentDistance, boolean byRank) throws Exception {
		if (byRank)
			this.updateSolutionKRank(currentWay,currentDistance);
		else
			this.updateSolutionKShortest(currentWay,currentDistance);
	}

	/**
	 * Is used by the algorithm of pathfinding to update the list of solutions.
	 * It works with the global structures and does a dichotomic insertion. The
	 * limit of the number of solutions is considered in the end of this
	 * procedure (the worstsolution is removed).
	 * @throws Exception
	 */
	synchronized void updateSolutionKShortest(List currentWay, float currentDistance ) throws Exception {
		try{
		Result res = new Result(currentWay, currentDistance);
		Result last;
		int n = this._tempsolutions.size();
		int pos = 0;
		int inf = 0;
		int sup = n;

		while (sup - inf >= 1) {
			last = (Result) this._tempsolutions.get(pos);

			if (currentDistance < last.getScore()) {
				sup = pos;
			} else {
				inf = pos + 1;
			}

			pos = (inf + sup) / 2;
		}

		this._tempsolutions.add(pos, res);

		n++;

		if (n > this._nbSolutions) {
			this._tempsolutions.remove(n - 1);

			last = (Result) this._tempsolutions.get(n - 2);

			this.setMaxDistance(last.getScore());

			// this.getCallingPathfinding().runrea();
		}

		if ((this.getTimeout() > -1)
				&& (System.currentTimeMillis() - this.getStarttime() > this
						.getTimeout())) {
			if (!this.isOnlyBacktrack()) {
				if (n < this._nbSolutions) { // if the required
					// number of solution
					// have not been reach
					// give it a little
					// more time
					System.out.println("nbSolution = " + (n - 1));
					this.setStarttime(System.currentTimeMillis());
				} else { // else say to the
					// pathfinding it can
					// interrupt this
					// thread and go to
					// REA. There is
					// enough solution to
					// find
					this.printResults(_tempsolutions.iterator()); // print the
					// solutions
					// already founded
					System.out.println("nbSolution = " + (n - 1)
							+ " notifing....");
					this.getCallingPathfinding().notifyBacktrackEnd(); 
					// I had
					// enough
					// lets
					// REA
				}
			}else
				this.getCallingPathfinding().notifyBacktrackEnd();
		}
		}catch(Exception e){
			e.printStackTrace();
			System.out.println(e.getMessage());
			throw e;
		}
	}

	protected synchronized void updateSolutionKRank(List currentWay, float currentDistance ) {

		Result res = new Result(currentWay, currentDistance);
		Result last;
		Float n = null;
		Float currentdistance = new Float(currentDistance);
		;
		ArrayList pathwaylist = null;
		if (!this._tempsolutionsmap.isEmpty()) {
			n = (Float) this._tempsolutionsmap.lastKey();
			pathwaylist = (ArrayList) this._tempsolutionsmap
					.get(currentdistance);
		}
		if (pathwaylist == null) {
			if (n == null || n.compareTo(currentdistance) > 0) {
				pathwaylist = new ArrayList();
				pathwaylist.add(res);
				this._tempsolutionsmap.put(currentdistance, pathwaylist);
				if (this._tempsolutionsmap.size() > this._nbSolutions) {
					this._tempsolutionsmap.remove(this._tempsolutionsmap
							.lastKey());

				}
				this.setMaxDistance(((Float) this._tempsolutionsmap.lastKey())
						.floatValue());

			}
		} else {
			pathwaylist.add(res);
		}

		if ((this.getTimeout() > -1)
				&& (System.currentTimeMillis() - this.getStarttime() > this
						.getTimeout())) {
			if (!this.isOnlyBacktrack()) {
				int size = this._tempsolutionsmap.size();
				if ((size < this._nbSolutions)) { // if the required
					// number of rank
					// have not been reach
					// give it a little
					// more time
					System.out.println("nbSolution = " + (size - 1));
					this.setStarttime(System.currentTimeMillis());
				} else { // else say to the
					// pathfinding it can
					// interrupt this
					// thread and go to
					// REA. There is
					// enough solution to
					// find
					this.printResults(_tempsolutions.iterator()); // print the
					// solutions
					// already founded
					System.out.println("nbSolution = " + (size - 1)
							+ " notifing....");
					this.getCallingPathfinding().notifyBacktrackEnd(); // I had
					// enough
					// lets
					// REA
				}
			} else
				this.getCallingPathfinding().notifyBacktrackEnd();
		}
	}

	/**
	 * Sets the number of solutions to obtain.
	 * 
	 * @param nbSolutions
	 *            the number of solutions.
	 */
	public synchronized void setNbSolutions(int nbSolutions) {
		_nbSolutions = nbSolutions;
	}

	/**
	 * Returns the number of solutions to obtain.
	 * 
	 * @return the number of solutions.
	 */
	public synchronized int getNbSolutions() {
		return _nbSolutions;
	}
	
	/**
	 * Sets the maximum level for the research.
	 * 
	 * @param maxLevel
	 *            the maximum level value.
	 */
	protected synchronized void setMaxLevel(int maxLevel) {
		_maxLevel = maxLevel;
	}

	/**
	 * Returns the maximum level for the research.
	 * 
	 * @return the maximum level value.
	 */
	protected synchronized int getMaxLevel() {
		return _maxLevel;
	}

	/**
	 * Sets the maximum distance for the research.
	 * 
	 * @param maxDistance
	 *            the maximum distance value.
	 */
	protected synchronized void setMaxDistance(float maxDistance) {
		_maxDistance = maxDistance;
	}

	/**
	 * Returns the the maximum distance for the research.
	 * 
	 * @return the maximum distance value.
	 */
	protected synchronized float getMaxDistance() {
		return _maxDistance;
	}
	public synchronized long getTimeout() {
		return _timeout;
	}

	/**
	 * @param l
	 */
	public synchronized void setTimeout(long l) {
		_timeout = l;
	}

	/**
	 * @return
	 */
	public synchronized long getStarttime() {
		return _starttime;
	}

	/**
	 * @param l
	 */
	public synchronized void setStarttime(long l) {
		_starttime = l;
	}

	/**
	 * @return
	 */
	public synchronized Pathfinding getCallingPathfinding() {
		return callingPathfinding;
	}

	/**
	 * @param pathfinding
	 */
	public synchronized void setCallingPathfinding(Pathfinding pathfinding) {
		callingPathfinding = pathfinding;
	}

	public synchronized ArrayList getTempSolutions() {
		return _tempsolutions;
	}
	public Solutions getSolutions(){
		Solutions sol = new Solutions();
		sol.addPathways(_tempsolutions);
		return sol;
	}
	public synchronized void setTempSolutions(ArrayList _tempsolutions) {
		this._tempsolutions = _tempsolutions;
	}
	public synchronized TreeMap getTempSolutionsMap() {
		return _tempsolutionsmap;
	}
	public synchronized void setTempSolutionsMap(TreeMap _tempsolutionsmap) {
		this._tempsolutionsmap = _tempsolutionsmap;
	}
	/**
	 * @return Returns the onlyBacktrack.
	 */
	public synchronized boolean isOnlyBacktrack() {
		return onlyBacktrack;
	}

	/**
	 * @param onlyBacktrack
	 *            The onlyBacktrack to set.
	 */
	public synchronized void setOnlyBacktrack(boolean onlyBacktrack) {
		this.onlyBacktrack = onlyBacktrack;
	}
	/**
	 * printResults print a Result arraylist iterator for debugging
	 * 
	 * @param itr
	 *            the iterator of a Result arrayList
	 */
	public synchronized void printResults(Iterator itr) {

		int i = 0;

		while (itr.hasNext()) {
			Result res = (Result) itr.next();
			Iterator pathItr = res.getPath().iterator();

			System.out.println("Pathway distance: " + res.getScore());

			while (pathItr.hasNext()) {
				System.out.print(pathItr.next() + "--->");
			}

			System.out.println();
		}
	}
}
