package graph.weighted.weightedNodes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

/**
 * Contains the definition and the methods for a weighted graph
 *
 * @author Fabian Couche
 * @version $Revision: 1.9 $
 */
public class Graph {

	/*
	 * This is the main structures of the graph. The ArrayList 'elements' contains
	 * the Nodes (the list 'nxt' of a Node is composed by the positions of the next
	 * nodes in the ArrayList 'elements' )
	 */

	/** Field elements */
	private ArrayList _elements;

	/** Field compounds */
	private HashMap _compounds;

	/** Field reactions */
	private HashMap _reactions;

	/** Field pos */
	private int _pos = 0;

	/* This is the global structures needed for the pathfinding */

	/** Field solutions */
	private ArrayList _solutions;

	/** Field nbSolutions */
	private int _nbSolutions;

	/** Field limit */
	private float _limit = 1000;

	/** Field currentWay */
	private LinkedList _currentWay;

	/** Field currentDistance */
	private float _currentDistance;

	/** Field end */
	private String _end;

	/** Field level */
	private int _level = 0;

	/** Field levelMax */
	private int _levelMax = 9;

	/** Field source */
	private int _source;

	/** Field timeOver */
	private boolean _timeOver;

	/** Field startTime */
	private long _startTime;

	/** Field timeout */
	private long _timeout = 300000;

	/* This is the structures which are needed to do the REA method */

	/** Field C[] */
	private LinkedList _c[];

	/** Field PI[] */
	private LinkedList _pi[];

	/** Field C2[] */
	private LinkedList _c2[];

	/** Field PI2[] */
	private LinkedList _pi2[];

	/** Field C3[] */
	private LinkedList _c3[];

	/** Field PI3[] */
	private LinkedList _pi3[];

	/** Field C4[] */
	private LinkedList _c4[];

	/** Field PI4[] */
	private LinkedList _pi4[];

	/** Field source2 */
	private int _source2;

	/** Field source3 */
	private int _source3;

	/** Field source4 */
	private int _source4;

	/**
	 * Graph Constructor
	 */
	public Graph() {

		_elements = new ArrayList();
		_compounds = new HashMap();
		_reactions = new HashMap();
	}

	/**
	 * Adds a new compound in the graph (creation of a new node unlinked).
	 * @param id A String for the identifier
	 * @param w A float for the weight
	 */
	public void addCompound(String id, float w) {

		// 
		assert id != null;

		// 
		Node n = new Node(id, w);
		Integer pos2 = new Integer(_pos);

		_elements.add(_pos, n);
		_compounds.put(n.getId(), pos2);

		++_pos;

		// tests
		if (!_compounds.containsKey(id)) {
			System.out.println("ERROR: bad insert of node!");
		}
	}

	/**
	 * keySetCompounds Method
	 * @return a set of compounds
	 */
	public Set keySetCompounds() {

		Set s = _compounds.keySet();

		return s;
	}

	/**
	 * Method to follow when a needed compound is not present.
	 * @param x the compound
	 */
	public void missingCompound(String x) {

		// 
		assert x != null;

		// 
		System.out.println("ERROR: compound not present!");
		System.exit(1);
	}

	/**
	   *  Adds a new reaction (and the reverse) in the graph. The parameters are
	   *  the Id and the weight of the reaction, the list of Id of substrates and
	   *  the list of Id of products.
	   * @param id A String for identifier
	   * @param w A float for the weight
	   * @param substrates A List of the substrates
	   * @param products A list of the products
	   */
	/* public void addAssociatedReaction (Properties prop, List substrates, List products){
	    
	
	    // 
	    assert prop != null;
	
	    assert substrates != null;
	
	    assert products != null;
	
	    Node direct = new Node();
	    // 
	
	    
	    
	    
	    
	    
	}*/

	/**
	 *  Adds a new reaction (and the reverse) in the graph. The parameters are
	 *  the Id and the weight of the reaction, the list of Id of substrates and
	 *  the list of Id of products.
	 * @param id A String for identifier
	 * @param w A float for the weight
	 * @param substrates A List of the substrates
	 * @param products A list of the products
	 * @return return the direct Node
	 */
	public Node addReaction(
		String id,
		float w,
		List substrates,
		List products) {

		// 
		assert id != null;

		//assert substrates != null; // reactions can have only product or only substrate in the case of excluded

		//assert products != null;

		// 

		Node direct = new Node(id, w);
		direct.setDirect(true);

		return addReaction(direct, substrates, products);

	}

	/**
	   *  Adds a new reaction (and the reverse) in the graph. The parameters are
	   *  are provided by the Node of the reaction, the list of Id of substrates and
	   *  the list of Id of products.
	   * @param direct the direct reaction  node;
	   * @param substrates A List of the substrates
	   * @param products A list of the products
	 * @return return the direct Node
	*/
	public Node addReaction(Node direct, List substrates, List products) {

		// 
		assert direct != null;

		assert substrates != null;

		assert products != null;

		// 
		String ident;
		Node n;
		Integer pos2, position, posDirect, posReverse;
		LinkedList couple = new LinkedList();

		posDirect = new Integer(_pos);

		_elements.add(_pos, direct);

		pos2 = new Integer(_pos);

		couple.addLast(pos2);

		++_pos;

		posReverse = new Integer(_pos);

		Node reverse = new Node(direct);
		reverse.setDirect(false);

		_elements.add(_pos, reverse);

		pos2 = new Integer(_pos);

		couple.addLast(pos2);

		++_pos;

		_reactions.put(direct.getId(), couple);

		if (products != null) {

			for (ListIterator i = products.listIterator(); i.hasNext();) {
				ident = (String) i.next();

				if (!_compounds.containsKey(ident)) {
					missingCompound(ident);
				}

				position = (Integer) _compounds.get(ident);

				direct.getNxt().add(position);

				n = (Node) _elements.get(position.intValue());

				n.getNxt().add(posReverse);
				n.getPrev().add(posDirect);
			}
		}
		if (substrates != null) {
			for (ListIterator i = substrates.listIterator(); i.hasNext();) {
				ident = (String) i.next();

				if (!_compounds.containsKey(ident)) {
					missingCompound(ident);
				}

				position = (Integer) _compounds.get(ident);

				reverse.getNxt().add(position);

				n = (Node) _elements.get(position.intValue());

				n.getNxt().add(posDirect);
				n.getPrev().add(posReverse);
			}
		}
		direct.setPrev(new LinkedList(reverse.getNxt()));
		reverse.setPrev(new LinkedList(direct.getNxt()));
		return direct;
	}

	/**
	 * Gets the position of the node in the graph.
	 * @param position the position of the node.
	 * @return the asked node.
	 */
	public Node getNodeByPos(int position) {
		return (Node) _elements.get(position);
	}

	/**
	 * Returns the reactions by the identifier.
	 * @param reactionId an identifier.
	 * @return the researched reactions.
	 */
	public LinkedList getReactionPositionsById(String reactionId) {
		return (LinkedList) _reactions.get(reactionId);
	}

	/**
	 * Returns the position of a compound.
	 * @param compoundId the id of the researched compound.
	 * @return the position.
	 */
	public int getCompoundPositionById(String compoundId) {

		Integer compoundPosition = (Integer) _compounds.get(compoundId);

		return compoundPosition.intValue();
	}

	/**
	 * Returns the position of a node in the graph.
	 * @param nodeId the id of the researched node.
	 * @return the position in the graph structure.
	 
	public Integer getCpdNodePositionById(String nodeId) {
		 return (Integer) _compounds.get(nodeId);
	}
*/
	/**
	 * Returns the position of a node.
	 * @param n the researched node.
	 * @return the position.
	 */
	public int getNodePosition(Node n) {
		return _elements.indexOf(n);
	}

	/**
	 * Returns the current number of elements.
	 * @return the number of elements
	 */
	public int getNumberElements() {
		return _pos;
	}

	/**
	 * Return true if the graph contains the given compound.
	 * @param compoundId the identifier for the compound.
	 * @return a boolean.
	 */
	public boolean containsCompoundId(String compoundId) {
		return _compounds.containsKey(compoundId);
	}

	/**
	 * Prints the list of elements.
	 * @param list the list to print.
	 */
	public void printList(LinkedList list) {
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			Object element = (Object) iter.next();
			System.out.print(element + " ");
		}
		System.out.println();
	}

	/**
	 * Prints the elements of the current graph.
	 */
	public void printElements() {

		ArrayList elements = this.getElements();

		for (Iterator iter = elements.iterator(); iter.hasNext();) {
			Node element = (Node) iter.next();

			System.out.println(element.getId());

			List successors = element.getNxt();

			System.out.print(" Successors:");

			for (Iterator iterator = successors.iterator();
				iterator.hasNext();
				) {
				Integer successorPos = (Integer) iterator.next();
				Node successor = this.getNodeByPos(successorPos.intValue());

				System.out.print(" " + successor.getId());
			}

			System.out.println("");

			List predecessors = element.getPrev();

			System.out.print(" Predecessors:");

			for (Iterator iterator = predecessors.iterator();
				iterator.hasNext();
				) {
				Integer predecessorPos = (Integer) iterator.next();
				Node predecessor = this.getNodeByPos(predecessorPos.intValue());

				System.out.print(" " + predecessor.getId());
			}

			System.out.println("");
		}
	}

	/**
	 *  Used by the algorithm of pathfinding to update
	 *  the list of solutions. It works with the global structures and does a
	 *  dichotomic insertion. The limit of the number of solutions is considered
	 *  in the end of this procedure (the worstsolution is removed).
	 */
	public void updateSolution() {

		Result res = new Result(_currentWay, _currentDistance);
		Result last;
		int n = _solutions.size();
		int pos = 0;
		int inf = 0;
		int sup = n;

		while (sup - inf >= 1) {
			last = (Result) _solutions.get(pos);

			if (_currentDistance < last.getScore()) {
				sup = pos;
			} else {
				inf = pos + 1;
			}

			pos = (inf + sup) / 2;
		}

		_solutions.add(pos, res);

		n++;

		if (n > _nbSolutions) {
			_solutions.remove(n - 1);

			last = (Result) _solutions.get(n - 2);
			_limit = last.getScore();
		}
	}

	/**
	 * Initialises the needed structures with the parameters and launch the research.
	 * 
	 * @param nb The number of pathways needed
	 * @param startID The source ID
	 * @param endID The destination ID
	 * @param weightM The maximum weight
	 * @param levelM The maximum level
	 * @param algo The number of the algorithm
	 *
	 * @return A List of solutions
	 * @deprecated replaced by the static method in Pathfinding Class 
	 */
	public List pathFinding(
		int nb,
		String startID,
		String endID,
		float weightM,
		int levelM,
		int algo) {

		// 
		assert startID != null;

		assert endID != null;

		// 
		Node startNode, startNode2;
		Integer startPos, startPos1, startPos2;

		_end = endID;
		_solutions = new ArrayList();
		_currentWay = new LinkedList();
		_limit = weightM;
		_level = 1;
		_levelMax = 1 + (levelM * 2);
		_nbSolutions = nb;
		_timeOver = false;

		if (!_compounds.containsKey(startID)) {
			LinkedList couple = (LinkedList) _reactions.get(startID);

			startPos1 = (Integer) couple.getFirst();
			startPos2 = (Integer) couple.getLast();
			startNode = (Node) _elements.get(startPos1.intValue());
			startNode2 = (Node) _elements.get(startPos2.intValue());

			switch (algo) {

				case 0 :
					_startTime = System.currentTimeMillis();
					_timeOver = false;

					_currentWay.add(0, startNode.getId());

					_currentDistance = startNode.getWeight();

					backtrak(startNode);

					_currentWay = new LinkedList();

					_currentWay.add(0, startNode2.getId());

					_currentDistance = startNode2.getWeight();

					backtrak(startNode2);
					break;

				case 1 :
					_currentWay.add(0, startNode.getId());

					_currentDistance = startNode.getWeight();

					rea(startNode, startNode2);
					break;

				case 2 :
					_currentWay.add(0, startNode.getId());

					_currentDistance = startNode.getWeight();

					rea2(startNode, startNode2);
					break;
			}
		} else {
			startPos = (Integer) _compounds.get(startID);
			startNode = (Node) _elements.get(startPos.intValue());

			_currentWay.add(0, startNode.getId());

			_currentDistance = startNode.getWeight();

			switch (algo) {

				case 0 :
					_startTime = System.currentTimeMillis();
					_timeOver = false;

					backtrak(startNode);
					break;

				case 1 :
					rea(startNode);
					break;

				case 2 :
					rea2(startNode);
					break;
			}
		}

		if (_timeOver) {
			_solutions.clear();
		}

		return _solutions;
	}

	/**
	 *
	 */
	public void cleanMarks() {

		for (ListIterator iter = _elements.listIterator(); iter.hasNext();) {
			Node n = (Node) iter.next();

			n.unmark();
		}
	}

	/**
	 * Verifies if the node has an outgoing link to the index position in the graph
	 * @param n A node
	 * @param pos A position in the index of the graph
	 * @return a boolean
	 */
	public boolean existLink(Node n, int pos) {

		// 
		assert n != null;

		// 
		int intValue;
		Integer integerValue;
		boolean found = false;

		for (ListIterator x = n.getNxt().listIterator(); x.hasNext();) {
			integerValue = (Integer) x.next();
			intValue = integerValue.intValue();

			if (intValue == pos) {
				found = true;

				break;
			}
		}

		return found;
	}

	/**
	 * Inserts in right position a Path in a list.
	 * @param k a path.
	 * @param l an arrayList.
	 */
	public void insert(Path k, ArrayList l) {

		// 
		assert k != null;

		assert l != null;

		// 
		Path last;
		int n = l.size();
		int pos = 0;
		int inf = 0;
		int sup = n;

		while (sup - inf >= 1) {
			last = (Path) l.get(pos);

			if (k.getLenght() < last.getLenght()) {
				sup = pos;
			} else {
				inf = pos + 1;
			}

			pos = (inf + sup) / 2;
		}

		l.add(pos, k);
	}

	/**
	 * Verifies the constraints
	 * @param p A Path
	 * @param n A Node
	 * @return a boolean
	 */
	public boolean verifyConstraints(Path p, Node n) {

		// 
		assert p != null;

		assert n != null;

		// 
		int cpt = 1;
		boolean test;
		boolean result = false;
		Node elem;
		String id = n.getId();
		float val = p.getLenght();

		val += n.getWeight();

		if (val < _limit) {
			do {
				test = true;
				elem = (Node) _elements.get(p.getNodeId());

				if (elem.getId() == id) {
					test = false;
				}

				p = p.getBack();

				cpt++;
			} while ((p.getBack() != p) && (cpt < _levelMax) && (test));

			if (p.getBack() == p) {
				result = true;
			}
		}

		return result;
	}

	/**
	 * An adapted version of the famous Dijkstra algorithm with constraints
	 * @param startNode A Node
	 */
	public void dijkstraWithConstraints(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		_pi = new LinkedList[taille];
		_pi[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;
				n = (Node) _elements.get(i);

				if ((existLink(startNode, i))) {
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi[i].add(k);

				if ((insertion) && verifyConstraints(s, n)) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi[futur].getFirst();

					if ((v < k.getLenght())
						&& verifyConstraints(minPath, futurNode)) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			if (fringe.isEmpty()) {
				finish = true;
			}
		}
	}

	/**
	 * An adapted version of the famous Dijkstra algorithm
	 * @param startNode the start point.
	 */
	public void dijkstra(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		_pi = new LinkedList[taille];
		_pi[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;

				if ((existLink(startNode, i))) {
					n = (Node) _elements.get(i);
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi[i].add(k);

				if (insertion) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi[futur].getFirst();

					if (v < k.getLenght()) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			if (fringe.isEmpty()) {
				finish = true;
			}
		}
	}

	/**
	 * REA Method
	 * @param startNode A Node
	 */
	public void rea(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer endPosition = (Integer) _compounds.get(_end);
		boolean accept = true;
		int t = endPosition.intValue();
		int n = _nbSolutions;
		long inTime;

		inTime = System.currentTimeMillis();

		Path last;

		dijkstra(startNode);
		System.out.print("P[] = ");
		//this.printList();
		System.out.print("D[] = ");
		//this.printList(_c);   

		accept = buildPath(t, 0);

		if (!accept) {
			n++;
		}

		_c = new LinkedList[_pos];
		this._c[_source] = new LinkedList();

		for (int k = 1; k < n; ++k) {
			nextPath(t, k);

			accept = buildPath(t, k);

			if (!accept) {
				n++;
			}

			last = (Path) _pi[t].get(k);

			if (last.getBack() == null) {
				k += n;
			}

			if (System.currentTimeMillis() - inTime >= _timeout) {
				_timeOver = true;
				k += n;
			}
		}

		System.out.println("Total number: " + n);
	}

	/**
	 * version of REA that uses Dijkstra with constraints
	 * @param startNode A Node
	 */
	public void rea2(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer endPosition = (Integer) _compounds.get(_end);
		boolean accept = true;
		int t = endPosition.intValue();
		int n = _nbSolutions;
		long inTime;

		inTime = System.currentTimeMillis();

		Path last;

		dijkstraWithConstraints(startNode);

		accept = buildPath(t, 0);

		if (!accept) {
			n++;
		}

		_c = new LinkedList[_pos];
		this._c[_source] = new LinkedList();

		for (int k = 1; k < n; ++k) {
			nextPath(t, k);

			accept = buildPath(t, k);

			if (!accept) {
				n++;
			}

			last = (Path) _pi[t].get(k);

			if (last.getBack() == null) {
				k += n;
			}

			if (System.currentTimeMillis() - inTime >= _timeout) {
				_timeOver = true;
				k += n;
			}
		}
	}

	/**
	 * nextPath Method
	 * @param v An int
	 * @param k An int
	 */
	public void nextPath(int v, int k) {

		Integer integerVal;
		Node n;
		int intVal, kp;
		Path p, temp;

		if (v != this._source) {
			if (k == 1) {
				n = (Node) _elements.get(v);
				this._c[v] = new LinkedList();

				for (ListIterator i = n.getPrev().listIterator();
					i.hasNext();
					) {
					integerVal = (Integer) i.next();
					intVal = integerVal.intValue();
					p = (Path) _pi[intVal].getFirst();
					temp = (Path) _pi[v].getFirst();

					if (p != temp.getBack()) {
						_c[v].add(p);
					}
				}
			}

			temp = (Path) _pi[v].get(k - 1);
			p = temp.getBack();

			if (p != null) {
				intVal = p.getNode();
				kp = _pi[intVal].indexOf(p);

				if ((kp + 1) == _pi[intVal].size()) {
					nextPath(intVal, kp + 1);
				}

				temp = (Path) _pi[intVal].get(kp + 1);

				if (temp.getBack() != null) {
					_c[v].add(temp);
				}
			}
		}

		if (_c[v].size() != 0) {
			Path min = (Path) _c[v].getFirst();
			float l;

			for (ListIterator i = _c[v].listIterator(); i.hasNext();) {
				temp = (Path) i.next();

				if (temp.getLenght() < min.getLenght()) {
					min = temp;
				}
			}

			l = min.getLenght();
			n = (Node) _elements.get(v);
			l += n.getWeight();
			temp = new Path(min, l, v);

			_pi[v].add(temp);
			_c[v].remove(min);
		} else {
			temp = new Path(v);

			_pi[v].add(temp);
		}
	}

	/**
	 * buildPath Method
	 * @param target the destination position.
	 * @param k the researched path.
	 * @return a boolean
	 */
	public boolean buildPath(int target, int k) {

		Node elem;
		LinkedList marked = new LinkedList();
		int posit;
		int depht = 0;

		_currentWay = new LinkedList();

		Path p;

		p = (Path) _pi[target].get(k);
		_currentDistance = p.getLenght();

		if (p.getLenght() < _limit) {
			do {
				posit = p.getNode();
				elem = (Node) _elements.get(posit);

				if (elem.marked()) {
					removeMark(marked);

					return false;
				} else {
					elem.mark();
					marked.add(elem);
					_currentWay.addFirst(elem.getId());

					p = p.getBack();

					depht++;
				}

				if (depht > _levelMax) {
					removeMark(marked);

					return false;
				}
			} while (p.getBack() != p);

			posit = p.getNode();
			elem = (Node) _elements.get(posit);

			if (!elem.marked()) {
				_currentWay.addFirst(elem.getId());

				_currentDistance += elem.getWeight();
			} else {
				removeMark(marked);

				return false;
			}

			updateSolution();
			removeMark(marked);

			return true;
		}

		return false;
	}

	/**
	 * REA Method
	 * @param startNode1 the first node.
	 * @param startNode2 the second node.
	 */
	public void rea(Node startNode1, Node startNode2) {

		// 
		assert startNode1 != null;

		assert startNode2 != null;

		// 
		boolean accept = true;
		LinkedList couple = (LinkedList) _reactions.get(_end);
		Integer endPosition1 = (Integer) couple.getFirst();
		Integer endPosition2 = (Integer) couple.getLast();
		int endPos1 = endPosition1.intValue();
		int endPos2 = endPosition2.intValue();
		int n = _nbSolutions;
		int k1 = 1;
		int k2 = 1;
		int k3 = 1;
		int k4 = 1;
		int min;
		float minVal;
		float niveau[] = new float[4];
		long inTime;

		inTime = System.currentTimeMillis();

		Path last, p;

		last = null;
		n += 3;

		dijkstra(startNode1);

		accept = buildPath(endPos1, 0);

		if (!accept) {
			n++;
		}

		dijkstra2(startNode2);

		accept = buildPath2(endPos1, 0);

		if (!accept) {
			n++;
		}

		dijkstra3(startNode1);

		accept = buildPath3(endPos2, 0);

		if (!accept) {
			n++;
		}

		dijkstra4(startNode2);

		accept = buildPath4(endPos2, 0);

		if (!accept) {
			n++;
		}

		_c = new LinkedList[_pos];
		this._c[_source] = new LinkedList();
		_c2 = new LinkedList[_pos];
		this._c2[_source2] = new LinkedList();
		_c3 = new LinkedList[_pos];
		this._c3[_source3] = new LinkedList();
		_c4 = new LinkedList[_pos];
		this._c4[_source4] = new LinkedList();

		while ((k1 + k2 + k3 + k4) < n) {
			p = (Path) _pi[endPos1].get(k1 - 1);
			niveau[0] = p.getLenght();
			p = (Path) _pi2[endPos1].get(k2 - 1);
			niveau[1] = p.getLenght();
			p = (Path) _pi3[endPos2].get(k3 - 1);
			niveau[2] = p.getLenght();
			p = (Path) _pi4[endPos2].get(k4 - 1);
			niveau[3] = p.getLenght();
			minVal = niveau[0];
			min = 0;

			for (int i = 1; i < 4; i++) {
				if (niveau[i] < minVal) {
					minVal = niveau[i];
					min = i;
				}
			}

			switch (min) {

				case 0 :
					nextPath(endPos1, k1);

					accept = buildPath(endPos1, k1);
					last = (Path) _pi[endPos1].get(k1);

					k1++;
					break;

				case 1 :
					nextPath2(endPos1, k2);

					accept = buildPath2(endPos1, k2);
					last = (Path) _pi2[endPos1].get(k2);

					k2++;
					break;

				case 2 :
					nextPath3(endPos2, k3);

					accept = buildPath3(endPos2, k3);
					last = (Path) _pi3[endPos2].get(k3);

					k3++;
					break;

				case 3 :
					nextPath4(endPos2, k4);

					accept = buildPath4(endPos2, k4);
					last = (Path) _pi4[endPos2].get(k4);

					k4++;
					break;
			}

			if (!accept) {
				n++;
			}

			if (last.getBack() == null) {
				k1 += n;
			}

			if (System.currentTimeMillis() - inTime >= _timeout) {
				_timeOver = true;
				k1 += n;
			}
		}

		System.out.println("Total number: " + n);
	}

	/* It's the version of REA that uses Dijkstra with constraints */

	/**
	 * REA2 Method
	 * @param startNode1 the first node.
	 * @param startNode2 the second node.
	 */
	public void rea2(Node startNode1, Node startNode2) {

		// 
		assert startNode1 != null;

		assert startNode2 != null;

		// 
		boolean accept = true;
		LinkedList couple = (LinkedList) _reactions.get(_end);
		Integer endPosition1 = (Integer) couple.getFirst();
		Integer endPosition2 = (Integer) couple.getLast();
		int endPos1 = endPosition1.intValue();
		int endPos2 = endPosition2.intValue();
		int n = _nbSolutions;
		int k1 = 1;
		int k2 = 1;
		int k3 = 1;
		int k4 = 1;
		int min;
		float minVal;
		float niveau[] = new float[4];
		long inTime;

		inTime = System.currentTimeMillis();

		Path last, p;

		last = null;
		n += 3;

		dijkstraWithConstraints(startNode1);

		accept = buildPath(endPos1, 0);

		if (!accept) {
			n++;
		}

		dijkstraWithConstraints2(startNode2);

		accept = buildPath2(endPos1, 0);

		if (!accept) {
			n++;
		}

		dijkstraWithConstraints3(startNode1);

		accept = buildPath3(endPos2, 0);

		if (!accept) {
			n++;
		}

		dijkstraWithConstraints4(startNode2);

		accept = buildPath4(endPos2, 0);

		if (!accept) {
			n++;
		}

		_c = new LinkedList[_pos];
		this._c[_source] = new LinkedList();
		_c2 = new LinkedList[_pos];
		this._c2[_source2] = new LinkedList();
		_c3 = new LinkedList[_pos];
		this._c3[_source3] = new LinkedList();
		_c4 = new LinkedList[_pos];
		this._c4[_source4] = new LinkedList();

		while ((k1 + k2 + k3 + k4) < n) {
			p = (Path) _pi[endPos1].get(k1 - 1);
			niveau[0] = p.getLenght();
			p = (Path) _pi2[endPos1].get(k2 - 1);
			niveau[1] = p.getLenght();
			p = (Path) _pi3[endPos2].get(k3 - 1);
			niveau[2] = p.getLenght();
			p = (Path) _pi4[endPos2].get(k4 - 1);
			niveau[3] = p.getLenght();
			minVal = niveau[0];
			min = 0;

			for (int i = 1; i < 4; i++) {
				if (niveau[i] < minVal) {
					minVal = niveau[i];
					min = i;
				}
			}

			switch (min) {

				case 0 :
					nextPath(endPos1, k1);

					accept = buildPath(endPos1, k1);
					last = (Path) _pi[endPos1].get(k1);

					k1++;
					break;

				case 1 :
					nextPath2(endPos1, k2);

					accept = buildPath2(endPos1, k2);
					last = (Path) _pi2[endPos1].get(k2);

					k2++;
					break;

				case 2 :
					nextPath3(endPos2, k3);

					accept = buildPath3(endPos2, k3);
					last = (Path) _pi3[endPos2].get(k3);

					k3++;
					break;

				case 3 :
					nextPath4(endPos2, k4);

					accept = buildPath4(endPos2, k4);
					last = (Path) _pi4[endPos2].get(k4);

					k4++;
					break;
			}

			if (!accept) {
				n++;
			}

			if (last.getBack() == null) {
				k1 += n;
			}

			if (System.currentTimeMillis() - inTime >= _timeout) {
				_timeOver = true;
				k1 += n;
			}
		}

		// System.out.println("Total number: "+n);
	}

	/**
	 * dijkstraWithConstraints2 Method
	 * @param startNode the start point.
	 */
	public void dijkstraWithConstraints2(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source2 = startPos;

		Integer startPosition = new Integer(startPos);

		// Integer endPosition= (Integer) compounds.get(end);
		// int endPos= endPosition.intValue();
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi2 = new LinkedList[taille];
		_pi2[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi2[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;
				n = (Node) _elements.get(i);

				if ((existLink(startNode, i))) {
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi2[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi2[i].add(k);

				if ((insertion) && verifyConstraints(s, n)) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi2[futur].getFirst();

					if ((v < k.getLenght())
						&& verifyConstraints(minPath, futurNode)) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * dijkstra2 Method
	 * @param startNode the start point.
	 */
	public void dijkstra2(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source2 = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi2 = new LinkedList[taille];
		_pi2[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi2[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;

				if ((existLink(startNode, i))) {
					n = (Node) _elements.get(i);
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi2[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi2[i].add(k);

				if (insertion) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi2[futur].getFirst();

					if (v < k.getLenght()) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * nextPath2 Method
	 * @param v the node.
	 * @param k the current position.
	 */
	public void nextPath2(int v, int k) {

		Integer integerVal;
		Node n;
		int intVal, kp;
		Path p, temp;

		if (v != this._source2) {
			if (k == 1) {
				n = (Node) _elements.get(v);
				this._c2[v] = new LinkedList();

				for (ListIterator i = n.getPrev().listIterator();
					i.hasNext();
					) {
					integerVal = (Integer) i.next();
					intVal = integerVal.intValue();
					p = (Path) _pi2[intVal].getFirst();
					temp = (Path) _pi2[v].getFirst();

					if (p != temp.getBack()) {
						_c2[v].add(p);
					}
				}
			}

			temp = (Path) _pi2[v].get(k - 1);
			p = temp.getBack();

			if (p != null) {
				intVal = p.getNode();
				kp = _pi2[intVal].indexOf(p);

				if ((kp + 1) == _pi2[intVal].size()) {
					nextPath2(intVal, kp + 1);
				}

				temp = (Path) _pi2[intVal].get(kp + 1);

				if (temp.getBack() != null) {
					_c2[v].add(temp);
				}
			}
		}

		if (_c2[v].size() != 0) {
			Path min = (Path) _c2[v].getFirst();
			float l;

			for (ListIterator i = _c2[v].listIterator(); i.hasNext();) {
				temp = (Path) i.next();

				if (temp.getLenght() < min.getLenght()) {
					min = temp;
				}
			}

			l = min.getLenght();
			n = (Node) _elements.get(v);
			l += n.getWeight();
			temp = new Path(min, l, v);

			_pi2[v].add(temp);
			_c2[v].remove(min);
		} else {
			temp = new Path(v);

			_pi2[v].add(temp);
		}
	}

	/**
	 * buildPath2 Method
	 * @param target the destination node.
	 * @param k the researched path.
	 * @return the status.
	 */
	public boolean buildPath2(int target, int k) {

		Node elem;
		LinkedList marked = new LinkedList();
		int posit;
		int depht = 0;

		_currentWay = new LinkedList();

		Path p;

		p = (Path) _pi2[target].get(k);
		_currentDistance = p.getLenght();

		if (p.getLenght() < _limit) {
			do {
				posit = p.getNode();
				elem = (Node) _elements.get(posit);

				if (elem.marked()) {
					removeMark(marked);

					return false;
				} else {
					elem.mark();
					marked.add(elem);
					_currentWay.addFirst(elem.getId());

					p = p.getBack();

					depht++;
				}

				if (depht > _levelMax) {
					removeMark(marked);

					return false;
				}
			} while (p.getBack() != p);

			posit = p.getNode();
			elem = (Node) _elements.get(posit);

			if (!elem.marked()) {
				_currentWay.addFirst(elem.getId());

				_currentDistance += elem.getWeight();
			} else {
				removeMark(marked);

				return false;
			}

			updateSolution();

			// System.out.println(k+"th path ACCEPTED");
			removeMark(marked);

			return true;
		}

		return false;
	}

	/**
	 * dijkstraWithConstraints3 Method
	 * @param startNode the start point.
	 */
	public void dijkstraWithConstraints3(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source3 = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi3 = new LinkedList[taille];
		_pi3[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi3[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;
				n = (Node) _elements.get(i);

				if ((existLink(startNode, i))) {
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi3[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi3[i].add(k);

				if ((insertion) && verifyConstraints(s, n)) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi3[futur].getFirst();

					if ((v < k.getLenght())
						&& verifyConstraints(minPath, futurNode)) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * dijkstra3 Method
	 * @param startNode the start point.
	 */
	public void dijkstra3(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source3 = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi3 = new LinkedList[taille];
		_pi3[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi3[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;

				if ((existLink(startNode, i))) {
					n = (Node) _elements.get(i);
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi3[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi3[i].add(k);

				if (insertion) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi3[futur].getFirst();

					if (v < k.getLenght()) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * nextPath3 Method
	 * @param v the node.
	 * @param k the current position.
	 */
	public void nextPath3(int v, int k) {

		Integer integerVal;
		Node n;
		int intVal, kp;
		Path p, temp;

		if (v != this._source3) {
			if (k == 1) {
				n = (Node) _elements.get(v);
				this._c3[v] = new LinkedList();

				for (ListIterator i = n.getPrev().listIterator();
					i.hasNext();
					) {
					integerVal = (Integer) i.next();
					intVal = integerVal.intValue();
					p = (Path) _pi3[intVal].getFirst();
					temp = (Path) _pi3[v].getFirst();

					if (p != temp.getBack()) {
						_c3[v].add(p);
					}
				}
			}

			temp = (Path) _pi3[v].get(k - 1);
			p = temp.getBack();

			if (p != null) {
				intVal = p.getNode();
				kp = _pi3[intVal].indexOf(p);

				if ((kp + 1) == _pi3[intVal].size()) {
					nextPath3(intVal, kp + 1);
				}

				temp = (Path) _pi3[intVal].get(kp + 1);

				if (temp.getBack() != null) {
					_c3[v].add(temp);
				}
			}
		}

		if (_c3[v].size() != 0) {
			Path min = (Path) _c3[v].getFirst();
			float l;

			for (ListIterator i = _c3[v].listIterator(); i.hasNext();) {
				temp = (Path) i.next();

				if (temp.getLenght() < min.getLenght()) {
					min = temp;
				}
			}

			l = min.getLenght();
			n = (Node) _elements.get(v);
			l += n.getWeight();
			temp = new Path(min, l, v);

			_pi3[v].add(temp);
			_c3[v].remove(min);
		} else {
			temp = new Path(v);

			_pi3[v].add(temp);
		}
	}

	/**
	 * buildPath3 Method
	 * @param target the destination node.
	 * @param k the researched path.
	 * @return the status.
	 */
	public boolean buildPath3(int target, int k) {

		Node elem;
		LinkedList marked = new LinkedList();
		int posit;
		int depht = 0;

		_currentWay = new LinkedList();

		Path p;

		p = (Path) _pi3[target].get(k);
		_currentDistance = p.getLenght();

		if (p.getLenght() < _limit) {
			do {
				posit = p.getNode();
				elem = (Node) _elements.get(posit);

				if (elem.marked()) {
					removeMark(marked);

					return false;
				} else {
					elem.mark();
					marked.add(elem);
					_currentWay.addFirst(elem.getId());

					p = p.getBack();

					depht++;
				}

				if (depht > _levelMax) {
					removeMark(marked);

					return false;
				}
			} while (p.getBack() != p);

			posit = p.getNode();
			elem = (Node) _elements.get(posit);

			if (!elem.marked()) {
				_currentWay.addFirst(elem.getId());

				_currentDistance += elem.getWeight();
			} else {
				removeMark(marked);

				return false;
			}

			updateSolution();
			removeMark(marked);

			return true;
		}

		return false;
	}

	/**
	 * dijkstraWithConstraints4 Method
	 * @param startNode the start point.
	 */
	public void dijkstraWithConstraints4(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source4 = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi4 = new LinkedList[taille];
		_pi4[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi4[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;
				n = (Node) _elements.get(i);

				if ((existLink(startNode, i))) {
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi4[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi4[i].add(k);

				if ((insertion) && verifyConstraints(s, n)) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi4[futur].getFirst();

					if ((v < k.getLenght())
						&& verifyConstraints(minPath, futurNode)) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * dijkstra4 Method
	 * @param startNode the start point.
	 */
	public void dijkstra4(Node startNode) {

		// 
		assert startNode != null;

		// 
		Integer o; // variable of work
		Node n; // variable of work
		Path k, s;
		int taille = _pos;
		int cpt = taille - 1;
		int startPos = _elements.indexOf(startNode);

		_source4 = startPos;

		Integer startPosition = new Integer(startPos);
		Set m = new HashSet();
		ArrayList fringe = new ArrayList();
		long inTime, outTime;
		long initTime = 0;
		long minTime = 0;
		long succTime = 0;
		float dist;
		boolean insertion;

		// INITILISATION OF THE STRUCTURES
		inTime = System.currentTimeMillis();
		_pi4 = new LinkedList[taille];
		_pi4[startPos] = new LinkedList();
		s = new Path(startPos, 0, startPos);

		_pi4[startPos].add(s);
		m.add(startPosition);
		insert(s, fringe);

		for (int i = 0; i < taille; ++i) {
			if (i != startPos) {
				o = new Integer(i);
				insertion = false;

				if ((existLink(startNode, i))) {
					n = (Node) _elements.get(i);
					dist = n.getWeight();
					insertion = true;
				} else {
					dist = Integer.MAX_VALUE;
				}

				m.add(o);

				_pi4[i] = new LinkedList();
				k = new Path(s, dist, i);

				_pi4[i].add(k);

				if (insertion) {
					insert(k, fringe);
				}
			}
		}

		m.remove(startPosition);
		fringe.remove(startPosition);

		outTime = System.currentTimeMillis();
		initTime = outTime - inTime;

		System.out.println("Temps d'initialisation: " + initTime);

		// BEGINNING OF THE SEARCH
		boolean finish = false;
		Integer minId;
		Path minPath = null;
		float minVal;

		while (!finish) {

			// CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
			inTime = System.currentTimeMillis();
			minId = null;
			minVal = Integer.MAX_VALUE;
			minPath = (Path) fringe.get(0);
			minVal = minPath.getLenght();
			minId = new Integer(minPath.getNode());

			if (minVal == Integer.MAX_VALUE) {
				return;
			}

			m.remove(minId);
			fringe.remove(minPath);

			outTime = System.currentTimeMillis();
			minTime += (outTime - inTime);

			cpt--;

			// ANALYSE FOLLOWERS OF THE SELECTED NODE
			inTime = System.currentTimeMillis();
			n = (Node) _elements.get(minId.intValue());

			for (ListIterator i = n.getNxt().listIterator(); i.hasNext();) {
				Integer f = (Integer) i.next();
				int futur = f.intValue();

				if (m.contains(f)) {
					Node futurNode = (Node) _elements.get(futur);
					float v = minVal + futurNode.getWeight();

					k = (Path) _pi4[futur].getFirst();

					if (v < k.getLenght()) {
						k.setLenght(v);
						k.setBack(minPath);

						if (fringe.contains(k)) {
							fringe.remove(k);
						}

						insert(k, fringe);
					}
				}
			}

			outTime = System.currentTimeMillis();
			succTime += (outTime - inTime);

			if (fringe.isEmpty()) {
				finish = true;

				System.out.println("Temps de recherche du min: " + minTime);
				System.out.println(
					"Temps de traitement des successeurs: " + succTime);
			}
		}
	}

	/**
	 * nextPath4 Method
	 * @param v the node.
	 * @param k the current position.
	 */
	public void nextPath4(int v, int k) {

		Integer integerVal;
		Node n;
		int intVal, kp;
		Path p, temp;

		if (v != this._source4) {
			if (k == 1) {
				n = (Node) _elements.get(v);
				this._c4[v] = new LinkedList();

				for (ListIterator i = n.getPrev().listIterator();
					i.hasNext();
					) {
					integerVal = (Integer) i.next();
					intVal = integerVal.intValue();
					p = (Path) _pi4[intVal].getFirst();
					temp = (Path) _pi4[v].getFirst();

					if (p != temp.getBack()) {
						_c4[v].add(p);
					}
				}
			}

			temp = (Path) _pi4[v].get(k - 1);
			p = temp.getBack();

			if (p != null) {
				intVal = p.getNode();
				kp = _pi4[intVal].indexOf(p);

				if ((kp + 1) == _pi4[intVal].size()) {
					nextPath4(intVal, kp + 1);
				}

				temp = (Path) _pi4[intVal].get(kp + 1);

				if (temp.getBack() != null) {
					_c4[v].add(temp);
				}
			}
		}

		if (_c4[v].size() != 0) {
			Path min = (Path) _c4[v].getFirst();
			float l;

			for (ListIterator i = _c4[v].listIterator(); i.hasNext();) {
				temp = (Path) i.next();

				if (temp.getLenght() < min.getLenght()) {
					min = temp;
				}
			}

			l = min.getLenght();
			n = (Node) _elements.get(v);
			l += n.getWeight();
			temp = new Path(min, l, v);

			_pi4[v].add(temp);
			_c4[v].remove(min);
		} else {
			temp = new Path(v);

			_pi4[v].add(temp);
		}
	}

	/**
	 * buildPath4 Method
	 * @param target the destination node.
	 * @param k the researched path.
	 * @return the status.
	 * @return
	 */
	public boolean buildPath4(int target, int k) {

		Node elem;
		LinkedList marked = new LinkedList();
		int posit;
		int depht = 0;

		_currentWay = new LinkedList();

		Path p;

		p = (Path) _pi4[target].get(k);
		_currentDistance = p.getLenght();

		if (p.getLenght() < _limit) {
			do {
				posit = p.getNode();
				elem = (Node) _elements.get(posit);

				if (elem.marked()) {
					removeMark(marked);

					return false;
				} else {
					elem.mark();
					marked.add(elem);
					_currentWay.addFirst(elem.getId());

					p = p.getBack();

					depht++;
				}

				if (depht > _levelMax) {
					removeMark(marked);

					return false;
				}
			} while (p.getBack() != p);

			posit = p.getNode();
			elem = (Node) _elements.get(posit);

			if (!elem.marked()) {
				_currentWay.addFirst(elem.getId());

				_currentDistance += elem.getWeight();
			} else {
				removeMark(marked);

				return false;
			}

			updateSolution();
			removeMark(marked);

			return true;
		}

		return false;
	}

	/**
	 * removeMark Method
	 * @param marked the list to unmark.
	 */
	public void removeMark(LinkedList marked) {

		Node n;

		for (ListIterator i = marked.listIterator(); i.hasNext();) {
			n = (Node) i.next();

			n.unmark();
		}
	}

	/**
	 * Does a pathfinding using an optimised recursive algorithm of backtraking.
	 * @param currentNode A Node
	 */
	public void backtrak(Node currentNode) {

		// 
		assert currentNode != null;

		// 
		Node futurNode; // next node
		Integer futur; // position of next node

		if (currentNode.getId().equals(_end)) {
			if (System.currentTimeMillis() - _startTime > _timeout) {
				_timeOver = true;
			}

			updateSolution();
		} else {
			if (_level < _levelMax) {
				for (ListIterator i = currentNode.getNxt().listIterator();
					i.hasNext();
					) {
					futur = (Integer) i.next();
					futurNode = (Node) _elements.get(futur.intValue());

					if (!futurNode.marked()) {
						_currentDistance += futurNode.getWeight();

						if (_currentDistance < _limit) {
							++_level;

							currentNode.mark();
							_currentWay.addLast(futurNode.getId());
							backtrak(futurNode);
							_currentWay.removeLast();
							currentNode.unmark();

							--_level;
						}

						_currentDistance -= futurNode.getWeight();
					}

					if (_timeOver) {
						break;
					}
				}
			}
		}
	}

	/**
	 * getElements Method
	 * @return the elements.
	 */
	public ArrayList getElements() {
		return _elements;
	}

	/**
	 * setElements Method
	 * @param elements the elements.
	 */
	public void setElements(ArrayList elements) {
		this._elements = elements;
	}
}

/*--- Formatted in North Style ---*/
