package graph.weighted.weightedNodes;



import graph.weighted.GraphData;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import java.util.Set;

import junit.framework.TestCase;


/**
 * The PathFinding Test Unit.
 *
 * @author Fabian Couche
 *
 * @version $Revision: 1.2 $
 */
public class GraphTest
  extends TestCase {

  // //////////////////////////////////////////////////////////
  // 
  // Basic Operations
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Tests the PathFinding.
   * @throws Exception an exception.
   */
  public void testPathFinding ()
    throws Exception {

    Solutions sol = new Solutions ();
    String graphDataFile = new String ("metabolic.gdt");
    GraphData gData = new GraphData (graphDataFile);
    Graph g = gData.getGraph ();
    Set s = g.keySetCompounds ();
    int taille = s.size ();
    Object c [] = s.toArray ();
    Random randCompound = new Random ();
    int i;
    int id = 900;
    boolean timeOut;
    int timeLimit = 60000;
    long inTime = 0;
    long totalTime = 0;
    String sourceID, targetID;
    String line = null;
    FileWriter file = new FileWriter ("tests.txt", true);
    BufferedWriter buff = new BufferedWriter (file);

    do {
      i = Math.abs (randCompound.nextInt () % taille);
      sourceID = (String) c [i];
      i = Math.abs (randCompound.nextInt () % taille);
      targetID = (String) c [i];

      sol.setPathways ((ArrayList) g.pathFinding (1, sourceID, targetID, 500, 8, 2));
    } while (sol.isEmpty ());

    int nbSol = 10;
    int algo = 0;
    int nbReact = 2;
    float weightMax = 100;

    timeOut = false;
    inTime = System.currentTimeMillis ();

    sol.setPathways ((ArrayList) g.pathFinding (nbSol, sourceID, targetID, weightMax, nbReact, algo));

    totalTime = System.currentTimeMillis () - inTime;

    if (totalTime > timeLimit) {
      timeOut = true;
    } else if (!sol.isEmpty ()) {
      sol.textExport (id, "paths.txt");
    }

    if (timeOut) {
      totalTime = timeLimit;
    }

    line = new String (sourceID + "\t" + targetID + "\t" + algo + "\t" + nbReact + "\t" + weightMax + "\t" + nbSol
                       + "\t" + sol.getSize () + "\t" + timeOut + "\t" + totalTime);

    try {
      buff.write (line);
      buff.newLine ();
    } catch (IOException e) {
      e.printStackTrace ();
    }

    buff.close ();
  }
}


/*--- Formatted in North Style ---*/
