package graph.weighted.weightedNodes;



import java.util.LinkedList;
import java.util.List;


/**
 * Class: Node
 * Description: defines the structure of a node and the associated procedures.
 *
 * @author Fabian Couche
 * @version $Revision: 1.2 $
 */
public class Node {

  /** Field info */
  private Properties _info;

  /** Field nxt */
  private List _nxt;

  /** Field prev */
  private List _prev;
  boolean direct;

  /**
   * Creates the properties with the parameters and a empty list to can insert the components later.
   *
   * @param s the id.
   * @param w the weight.
   */
  public Node (String s, float w) {

    // 
    assert s != null;

    // 
    _info = new Properties (s, w);

    setNxt (new LinkedList ());
    setPrev (new LinkedList ());
  }

  /**
   * References the properties of the node in parameter (equal without a new thus the properties of the two nodes are the same object in memory.
   * This operation is usefull to gain memory and to mark the reaction and his reverse.
   *
   * @param inv a node with the same info.
   */
  public Node (Node inv) {

    // 
    assert inv != null;

    // 
    _info = inv._info;  // to verify because info is private

    setNxt (new LinkedList ());
  }

  /**
   * getId() Method
   *
   * @return the id of the node
   */
  public String getId () {

    // 
    assert _info.getId () != null;

    // 
    return _info.getId ();
  }

  /**
   * getWeight() Method
   *
   * @return the weight of the node
   */
  public float getWeight () {
    return _info.getWeight ();
  }

  /**
   * Sets the mark of node to true.
   */
  public void mark () {
    _info.mark (true);
  }

  /**
   * Sets the mark of node to false.
   */
  public void unmark () {
    _info.mark (false);
  }

  /**
   * Returns the  current mark of the node.
   * @return true if it's marked.
   */
  public boolean marked () {
    return _info.isMarked ();
  }

  /**
   * getInfo Method
   * @return the informations of the node.
   */
  public Properties getInfo () {
    return _info;
  }

  /**
   * Sets the successors.
   * @param nxt the list of the next nodes.
   */
  public void setNxt (List nxt) {
    _nxt = nxt;
  }

  /**
   * Returns the successors.
   * @return list of the next nodes.
   */
  public List getNxt () {
    return _nxt;
  }

  /**
   * Sets the predecessors.
   * @param prev the list of the previous nodes.
   */
  public void setPrev (List prev) {
    _prev = prev;
  }

  /**
   * Returns the predecessors.
   * @return list of the previous nodes.
   */
  public List getPrev () {
    return _prev;
  }
 
/**
 * @return Returns the direct.
 */
public boolean isDirect() {
	return direct;
}
/**
 * @param direct The direct to set.
 */
public void setDirect(boolean direct) {
	this.direct = direct;
}
}


/*--- Formatted in North Style ---*/
