package graph.weighted.weightedNodes;



/**
 * Class: Path
 * Description: defines the structure of a node and the associated procedures
 *
 * @author Fabian Couche
 * @version $Revision: 1.1 $
 */
public class Path {

  /**  */
  private Path _back;

  /**  */
  private float _lenght;

  /**  */
  private int _nodeId;

  /**
   * Path Constructor
   * @param id the identifier
   */
  public Path (int id) {

    _lenght = Integer.MAX_VALUE;

    setBack (null);
    setNodeId (id);
  }

  /**
   * Path Constructor
   * @param p the path.
   * @param x the lenght.
   * @param id the identifier.
   */
  public Path (Path p, float x, int id) {

    setBack (p);

    _lenght = x;

    setNodeId (id);
  }

  /**
   * Path Constructor
   * @param s the node.
   * @param x the lenght.
   * @param id the identifier.
   */
  public Path (int s, float x, int id) {

    setBack (this);

    _lenght = x;

    setNodeId (id);
  }

  /**
   * Returns the lenght.
   * @return the lenght of the path.
   */
  public float getLenght () {
    return (_lenght);
  }

  /**
   * Sets the lenght.
   * @param l the lenght of the path.
   */
  public void setLenght (float l) {
    _lenght = l;
  }

  /**
   * Returns the node id.
   * @return the node.
   */
  public int getNode () {
    return getNodeId ();
  }

  /**
   * Sets the back pointer.
   * @param back the back pointer.
   */
  public void setBack (Path back) {
    _back = back;
  }

  /**
   * Returns the back pointer.
   * @return the back pointer.
   */
  public Path getBack () {
    return _back;
  }

  /**
   * Sets the identifier.
   * @param nodeId the identifier.
   */
  public void setNodeId (int nodeId) {
    _nodeId = nodeId;
  }

  /**
   * Returns the identifier.
   * @return the identifier.
   */
  public int getNodeId () {
    return _nodeId;
  }
}


/*--- Formatted in North Style ---*/
