package graph.weighted.weightedNodes;



import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Set;

import EDU.oswego.cs.dl.util.concurrent.Callable;


/**
 * Recursive Enumeration Algorithm
 *
 * @author Fabian Couche
 * @version $Id: REA.java,v 1.8 2005/02/25 10:12:19 didier Exp $
 */
public class REA
  implements Callable {

  // //////////////////////////////////////////////////////////
  // 
  // Basic operations
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Calls the algorithm.
   * @return the solutions.
   * @throws Exception if not initialized.
   */
  public Object call ()
    throws Exception {

    if (!this._isInitialized) {
      throw new Exception ("REA was not initialized!");
    } else {
      if (_graph.containsCompoundId (_startPointId)) {
        System.out.println ("It's a compound");

        int startPos = _graph.getCompoundPositionById(_startPointId);

        _startNode = _graph.getNodeByPos (startPos);

        rea2 (_startNode);
      } else {
        LinkedList couple = (LinkedList) _graph.getReactionPositionsById (_startPointId);
        Integer startPos1 = (Integer) couple.getFirst ();
        Integer startPos2 = (Integer) couple.getLast ();
        Node startNode1 = (Node) _graph.getNodeByPos (startPos1.intValue ());
        Node startNode2 = (Node) _graph.getNodeByPos (startPos2.intValue ());

        _currentWay.add (0, startNode1.getId ());

        _currentDistance = startNode1.getWeight ();

        rea2 (startNode1, startNode2);
      }
    }

    return _solutions;
  }

  /**
   * Initializes all what is needed by the backtracking.
   * @param graph the used graph.
   * @param startPointId the start point.
   * @param endPointId the end point.
   * @param nbSolutions the number of solutions to obtain.
   * @param maxDistance the maximum distance.
   * @param maxLevel the maximum level.
   */
  public void init (Graph graph, String startPointId, String endPointId, int nbSolutions,boolean byRank, float maxDistance,
                          int maxLevel) {

    _graph = graph;
    _startPointId = startPointId;
    _endPointId = endPointId;
    _nbSolutions = nbSolutions;
    _maxDistance = maxDistance;
    _maxLevel = maxLevel;
    _pos = _graph.getNumberElements ();
    _solutions = new ArrayList ();
    _currentWay = new LinkedList ();
    this._isInitialized = true;
  }

  /**
   * Verifies if the node has an outgoing link to the index position in the graph
   * @param n A node
   * @param pos A position in the index of the graph
   * @return a boolean
   */
  public boolean existLink (Node n, int pos) {

    // 
    assert n != null;

    // 
    int intValue;
    Integer integerValue;
    boolean found = false;

    for (ListIterator x = n.getNxt ().listIterator (); x.hasNext (); ) {
      integerValue = (Integer) x.next ();
      intValue = integerValue.intValue ();

      if (intValue == pos) {
        found = true;

        break;
      }
    }

    return found;
  }

  /**
   *  Used by the algorithm of pathfinding to update
   *  the list of solutions. It works with the global structures and does a
   *  dichotomic insertion. The limit of the number of solutions is considered
   *  in the end of this procedure (the worstsolution is removed).
   */
  public void updateSolution () {

    Result res = new Result (_currentWay, _currentDistance);
    Result last;
    int n = _solutions.size ();
    int pos = 0;
    int inf = 0;
    int sup = n;

    while (sup - inf >= 1) {
      last = (Result) _solutions.get (pos);

      if (_currentDistance < last.getScore ()) {
        sup = pos;
      } else {
        inf = pos + 1;
      }

      pos = (inf + sup) / 2;
    }

    _solutions.add (pos, res);

    n++;

    if (n > _nbSolutions) {
      _solutions.remove (n - 1);

      last = (Result) _solutions.get (n - 2);
      _limit = last.getScore ();
    }
  }

  /**
   * Inserts in right position a Path in a list.
   * @param k a path.
   * @param l an arrayList.
   */
  public void insert (Path k, ArrayList l) {

    // 
    assert k != null;

    assert l != null;

    // 
    Path last;
    int n = l.size ();
    int pos = 0;
    int inf = 0;
    int sup = n;

    while (sup - inf >= 1) {
      last = (Path) l.get (pos);

      if (k.getLenght () < last.getLenght ()) {
        sup = pos;
      } else {
        inf = pos + 1;
      }

      pos = (inf + sup) / 2;
    }

    l.add (pos, k);
  }


  /**
   * Verifies the constraints
   * @param p A Path
   * @param n A Node
   * @return a boolean
   */
  public boolean verifyConstraints (Path p, Node n) {

    // 
    assert p != null;

    assert n != null;

    // 
    int cpt = 1;
    boolean test;
    boolean result = false;
    Node elem;
    String id = n.getId ();
    float val = p.getLenght ();

    val += n.getWeight ();

    if (val < _limit) {
      do {
        test = true;
        elem = (Node) _graph.getNodeByPos (p.getNodeId ());

        if (elem.getId () == id) {
          test = false;
        }

        p = p.getBack ();

        cpt++;
      } while ((p.getBack () != p) && (cpt < _levelMax) && (test));

      if (p.getBack () == p) {
        result = true;
      }
    }

    return result;
  }

  /**
   * removeMark Method
   * @param marked the list to unmark.
   */
  public void removeMark (LinkedList marked) {

    Node n;

    for (ListIterator i = marked.listIterator (); i.hasNext (); ) {
      n = (Node) i.next ();

      n.unmark ();
    }
  }

  /**
   * An adapted version of the famous Dijkstra algorithm with constraints
   * @param startNode A Node
   */
  public void dijkstraWithConstraints (Node startNode) {

    // 
    assert startNode != null;

    // 
    System.out.println ("Dijkstra");

    Integer o;  // variable of work
    Node n;     // variable of work
    Path k, s;
    int taille = _pos;
    int cpt = taille - 1;
    int startPos = _graph.getNodePosition (startNode);

    _source = startPos;

    Integer startPosition = new Integer (startPos);
    Set m = new HashSet ();
    ArrayList fringe = new ArrayList ();
    float dist;
    boolean insertion;

    // INITILISATION OF THE STRUCTURES
    _pi = new LinkedList [taille];
    _pi [startPos] = new LinkedList ();
    s = new Path (startPos, 0, startPos);

    _pi [startPos].add (s);
    m.add (startPosition);
    insert (s, fringe);

    for (int i = 0; i < taille; ++i) {
      if (i != startPos) {
        o = new Integer (i);
        insertion = false;
        n = (Node) _graph.getNodeByPos (i);

        if ((existLink (startNode, i))) {
          dist = n.getWeight ();
          insertion = true;
        } else {
          dist = Integer.MAX_VALUE;
        }

        m.add (o);

        _pi [i] = new LinkedList ();
        k = new Path (s, dist, i);

        _pi [i].add (k);

        if ((insertion) && verifyConstraints (s, n)) {
          insert (k, fringe);
        }
      }
    }

    m.remove (startPosition);
    fringe.remove (startPosition);

    // BEGINNING OF THE SEARCH
    boolean finish = false;
    Integer minId;
    Path minPath = null;
    float minVal;

    while (!finish) {

      // CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
      minId = null;
      minVal = Integer.MAX_VALUE;
      minPath = (Path) fringe.get (0);
      minVal = minPath.getLenght ();
      minId = new Integer (minPath.getNode ());
      
      if (minVal == Integer.MAX_VALUE) {
        return;
      }

      m.remove (minId);
      fringe.remove (minPath);

      cpt--;

      // ANALYSE FOLLOWERS OF THE SELECTED NODE
      n = (Node) _graph.getNodeByPos (minId.intValue ());

      for (ListIterator i = n.getNxt ().listIterator (); i.hasNext (); ) {
        Integer f = (Integer) i.next ();
        int futur = f.intValue ();

        if (m.contains (f)) {
          Node futurNode = (Node) _graph.getNodeByPos (futur);
          float v = minVal + futurNode.getWeight ();

          k = (Path) _pi [futur].getFirst ();

          if ((v < k.getLenght ()) && verifyConstraints (minPath, futurNode)) {
            k.setLenght (v);
            k.setBack (minPath);

            if (fringe.contains (k)) {
              fringe.remove (k);
            }

            insert (k, fringe);
          }
        }
      }

      if (fringe.isEmpty ()) {
        finish = true;
      }
    }
  }

  /**
   * dijkstraWithConstraints2 Method
   * @param startNode the start point.
   */
  public void dijkstraWithConstraints2 (Node startNode) {

    // 
    assert startNode != null;

    // 
    Integer o;  // variable of work
    Node n;     // variable of work
    Path k, s;
    int taille = _pos;
    int cpt = taille - 1;
    int startPos = _graph.getNodePosition (startNode);

    _source2 = startPos;

    Integer startPosition = new Integer (startPos);

    // Integer endPosition= (Integer) compounds.get(end);
    // int endPos= endPosition.intValue();
    Set m = new HashSet ();
    ArrayList fringe = new ArrayList ();
    long inTime, outTime;
    long initTime = 0;
    long minTime = 0;
    long succTime = 0;
    float dist;
    boolean insertion;

    // INITILISATION OF THE STRUCTURES
    inTime = System.currentTimeMillis ();
    _pi2 = new LinkedList [taille];
    _pi2 [startPos] = new LinkedList ();
    s = new Path (startPos, 0, startPos);

    _pi2 [startPos].add (s);
    m.add (startPosition);
    insert (s, fringe);

    for (int i = 0; i < taille; ++i) {
      if (i != startPos) {
        o = new Integer (i);
        insertion = false;
        n = (Node) _graph.getNodeByPos (i);

        if ((existLink (startNode, i))) {
          dist = n.getWeight ();
          insertion = true;
        } else {
          dist = Integer.MAX_VALUE;
        }

        m.add (o);

        _pi2 [i] = new LinkedList ();
        k = new Path (s, dist, i);

        _pi2 [i].add (k);

        if ((insertion) && verifyConstraints (s, n)) {
          insert (k, fringe);
        }
      }
    }

    m.remove (startPosition);
    fringe.remove (startPosition);

    outTime = System.currentTimeMillis ();
    initTime = outTime - inTime;

    System.out.println ("Temps d'initialisation: " + initTime);

    // BEGINNING OF THE SEARCH
    boolean finish = false;
    Integer minId;
    Path minPath = null;
    float minVal;

    while (!finish) {

      // CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
      inTime = System.currentTimeMillis ();
      minId = null;
      minVal = Integer.MAX_VALUE;
      minPath = (Path) fringe.get (0);
      minVal = minPath.getLenght ();
      minId = new Integer (minPath.getNode ());

      if (minVal == Integer.MAX_VALUE) {
        return;
      }

      m.remove (minId);
      fringe.remove (minPath);

      outTime = System.currentTimeMillis ();
      minTime += (outTime - inTime);

      cpt--;

      // ANALYSE FOLLOWERS OF THE SELECTED NODE
      inTime = System.currentTimeMillis ();
      n = (Node) _graph.getNodeByPos (minId.intValue ());

      for (ListIterator i = n.getNxt ().listIterator (); i.hasNext (); ) {
        Integer f = (Integer) i.next ();
        int futur = f.intValue ();

        if (m.contains (f)) {
          Node futurNode = (Node) _graph.getNodeByPos (futur);
          float v = minVal + futurNode.getWeight ();

          k = (Path) _pi2 [futur].getFirst ();

          if ((v < k.getLenght ()) && verifyConstraints (minPath, futurNode)) {
            k.setLenght (v);
            k.setBack (minPath);

            if (fringe.contains (k)) {
              fringe.remove (k);
            }

            insert (k, fringe);
          }
        }
      }

      outTime = System.currentTimeMillis ();
      succTime += (outTime - inTime);

      if (fringe.isEmpty ()) {
        finish = true;

        System.out.println ("Temps de recherche du min: " + minTime);
        System.out.println ("Temps de traitement des successeurs: " + succTime);
      }
    }
  }

  /**
   * dijkstraWithConstraints3 Method
   * @param startNode the start point.
   */
  public void dijkstraWithConstraints3 (Node startNode) {

    // 
    assert startNode != null;

    // 
    Integer o;  // variable of work
    Node n;     // variable of work
    Path k, s;
    int taille = _pos;
    int cpt = taille - 1;
    int startPos = _graph.getNodePosition (startNode);

    _source3 = startPos;

    Integer startPosition = new Integer (startPos);
    Set m = new HashSet ();
    ArrayList fringe = new ArrayList ();
    long inTime, outTime;
    long initTime = 0;
    long minTime = 0;
    long succTime = 0;
    float dist;
    boolean insertion;

    // INITILISATION OF THE STRUCTURES
    inTime = System.currentTimeMillis ();
    _pi3 = new LinkedList [taille];
    _pi3 [startPos] = new LinkedList ();
    s = new Path (startPos, 0, startPos);

    _pi3 [startPos].add (s);
    m.add (startPosition);
    insert (s, fringe);

    for (int i = 0; i < taille; ++i) {
      if (i != startPos) {
        o = new Integer (i);
        insertion = false;
        n = (Node) _graph.getNodeByPos (i);

        if ((existLink (startNode, i))) {
          dist = n.getWeight ();
          insertion = true;
        } else {
          dist = Integer.MAX_VALUE;
        }

        m.add (o);

        _pi3 [i] = new LinkedList ();
        k = new Path (s, dist, i);

        _pi3 [i].add (k);

        if ((insertion) && verifyConstraints (s, n)) {
          insert (k, fringe);
        }
      }
    }

    m.remove (startPosition);
    fringe.remove (startPosition);

    outTime = System.currentTimeMillis ();
    initTime = outTime - inTime;

    System.out.println ("Temps d'initialisation: " + initTime);

    // BEGINNING OF THE SEARCH
    boolean finish = false;
    Integer minId;
    Path minPath = null;
    float minVal;

    while (!finish) {

      // CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
      inTime = System.currentTimeMillis ();
      minId = null;
      minVal = Integer.MAX_VALUE;
      minPath = (Path) fringe.get (0);
      minVal = minPath.getLenght ();
      minId = new Integer (minPath.getNode ());

      if (minVal == Integer.MAX_VALUE) {
        return;
      }

      m.remove (minId);
      fringe.remove (minPath);

      outTime = System.currentTimeMillis ();
      minTime += (outTime - inTime);

      cpt--;

      // ANALYSE FOLLOWERS OF THE SELECTED NODE
      inTime = System.currentTimeMillis ();
      n = (Node) _graph.getNodeByPos (minId.intValue ());

      for (ListIterator i = n.getNxt ().listIterator (); i.hasNext (); ) {
        Integer f = (Integer) i.next ();
        int futur = f.intValue ();

        if (m.contains (f)) {
          Node futurNode = (Node) _graph.getNodeByPos (futur);
          float v = minVal + futurNode.getWeight ();

          k = (Path) _pi3 [futur].getFirst ();

          if ((v < k.getLenght ()) && verifyConstraints (minPath, futurNode)) {
            k.setLenght (v);
            k.setBack (minPath);

            if (fringe.contains (k)) {
              fringe.remove (k);
            }

            insert (k, fringe);
          }
        }
      }

      outTime = System.currentTimeMillis ();
      succTime += (outTime - inTime);

      if (fringe.isEmpty ()) {
        finish = true;

        System.out.println ("Temps de recherche du min: " + minTime);
        System.out.println ("Temps de traitement des successeurs: " + succTime);
      }
    }
  }

  /**
   * dijkstraWithConstraints4 Method
   * @param startNode the start point.
   */
  public void dijkstraWithConstraints4 (Node startNode) {

    // 
    assert startNode != null;

    // 
    Integer o;  // variable of work
    Node n;     // variable of work
    Path k, s;
    int taille = _pos;
    int cpt = taille - 1;
    int startPos = _graph.getNodePosition (startNode);

    _source4 = startPos;

    Integer startPosition = new Integer (startPos);
    Set m = new HashSet ();
    ArrayList fringe = new ArrayList ();
    long inTime, outTime;
    long initTime = 0;
    long minTime = 0;
    long succTime = 0;
    float dist;
    boolean insertion;

    // INITILISATION OF THE STRUCTURES
    inTime = System.currentTimeMillis ();
    _pi4 = new LinkedList [taille];
    _pi4 [startPos] = new LinkedList ();
    s = new Path (startPos, 0, startPos);

    _pi4 [startPos].add (s);
    m.add (startPosition);
    insert (s, fringe);

    for (int i = 0; i < taille; ++i) {
      if (i != startPos) {
        o = new Integer (i);
        insertion = false;
        n = (Node) _graph.getNodeByPos (i);

        if ((existLink (startNode, i))) {
          dist = n.getWeight ();
          insertion = true;
        } else {
          dist = Integer.MAX_VALUE;
        }

        m.add (o);

        _pi4 [i] = new LinkedList ();
        k = new Path (s, dist, i);

        _pi4 [i].add (k);

        if ((insertion) && verifyConstraints (s, n)) {
          insert (k, fringe);
        }
      }
    }

    m.remove (startPosition);
    fringe.remove (startPosition);

    outTime = System.currentTimeMillis ();
    initTime = outTime - inTime;

    System.out.println ("Temps d'initialisation: " + initTime);

    // BEGINNING OF THE SEARCH
    boolean finish = false;
    Integer minId;
    Path minPath = null;
    float minVal;

    while (!finish) {

      // CHOICE OF THE NEXT NODE TO INSERT IN SOLUTION
      inTime = System.currentTimeMillis ();
      minId = null;
      minVal = Integer.MAX_VALUE;
      minPath = (Path) fringe.get (0);
      minVal = minPath.getLenght ();
      minId = new Integer (minPath.getNode ());

      if (minVal == Integer.MAX_VALUE) {
        return;
      }

      m.remove (minId);
      fringe.remove (minPath);

      outTime = System.currentTimeMillis ();
      minTime += (outTime - inTime);

      cpt--;

      // ANALYSE FOLLOWERS OF THE SELECTED NODE
      inTime = System.currentTimeMillis ();
      n = (Node) _graph.getNodeByPos (minId.intValue ());

      for (ListIterator i = n.getNxt ().listIterator (); i.hasNext (); ) {
        Integer f = (Integer) i.next ();
        int futur = f.intValue ();

        if (m.contains (f)) {
          Node futurNode = (Node) _graph.getNodeByPos (futur);
          float v = minVal + futurNode.getWeight ();

          k = (Path) _pi4 [futur].getFirst ();

          if ((v < k.getLenght ()) && verifyConstraints (minPath, futurNode)) {
            k.setLenght (v);
            k.setBack (minPath);

            if (fringe.contains (k)) {
              fringe.remove (k);
            }

            insert (k, fringe);
          }
        }
      }

      outTime = System.currentTimeMillis ();
      succTime += (outTime - inTime);

      if (fringe.isEmpty ()) {
        finish = true;

        System.out.println ("Temps de recherche du min: " + minTime);
        System.out.println ("Temps de traitement des successeurs: " + succTime);
      }
    }
  }

  /**
   * nextPath Method
   * @param v An int
   * @param k An int
   */
  public void nextPath (int v, int k) {

    System.out.println ("Searching next path.");

    Integer integerVal;
    Node n;
    int intVal, kp;
    Path p, temp;

    if (v != this._source) {
      if (k == 1) {
        n = (Node) _graph.getNodeByPos (v);
        this._c [v] = new LinkedList ();

        for (ListIterator i = n.getPrev ().listIterator (); i.hasNext (); ) {
          integerVal = (Integer) i.next ();
          intVal = integerVal.intValue ();
          p = (Path) _pi [intVal].getFirst ();
          temp = (Path) _pi [v].getFirst ();

          if (p != temp.getBack ()) {
            _c [v].add (p);
          }
        }
      }

      temp = (Path) _pi [v].get (k - 1);
      p = temp.getBack ();

      if (p != null) {
        intVal = p.getNode ();
        kp = _pi [intVal].indexOf (p);

        if ((kp + 1) == _pi [intVal].size ()) {
          nextPath (intVal, kp + 1);
        }

        temp = (Path) _pi [intVal].get (kp + 1);

        if (temp.getBack () != null) {
          _c [v].add (temp);
        }
      }
    }

    if (_c [v].size () != 0) {
      Path min = (Path) _c [v].getFirst ();
      float l;

      for (ListIterator i = _c [v].listIterator (); i.hasNext (); ) {
        temp = (Path) i.next ();

        if (temp.getLenght () < min.getLenght ()) {
          min = temp;
        }
      }

      l = min.getLenght ();
      n = (Node) _graph.getNodeByPos (v);
      l += n.getWeight ();
      temp = new Path (min, l, v);

      _pi [v].add (temp);
      _c [v].remove (min);
    } else {
      temp = new Path (v);

      _pi [v].add (temp);
    }
  }

  /**
   * nextPath2 Method
   * @param v the node.
   * @param k the current position.
   */
  public void nextPath2 (int v, int k) {

    Integer integerVal;
    Node n;
    int intVal, kp;
    Path p, temp;

    if (v != this._source2) {
      if (k == 1) {
        n = (Node) _graph.getNodeByPos (v);
        this._c2 [v] = new LinkedList ();

        for (ListIterator i = n.getPrev ().listIterator (); i.hasNext (); ) {
          integerVal = (Integer) i.next ();
          intVal = integerVal.intValue ();
          p = (Path) _pi2 [intVal].getFirst ();
          temp = (Path) _pi2 [v].getFirst ();

          if (p != temp.getBack ()) {
            _c2 [v].add (p);
          }
        }
      }

      temp = (Path) _pi2 [v].get (k - 1);
      p = temp.getBack ();

      if (p != null) {
        intVal = p.getNode ();
        kp = _pi2 [intVal].indexOf (p);

        if ((kp + 1) == _pi2 [intVal].size ()) {
          nextPath2 (intVal, kp + 1);
        }

        temp = (Path) _pi2 [intVal].get (kp + 1);

        if (temp.getBack () != null) {
          _c2 [v].add (temp);
        }
      }
    }

    if (_c2 [v].size () != 0) {
      Path min = (Path) _c2 [v].getFirst ();
      float l;

      for (ListIterator i = _c2 [v].listIterator (); i.hasNext (); ) {
        temp = (Path) i.next ();

        if (temp.getLenght () < min.getLenght ()) {
          min = temp;
        }
      }

      l = min.getLenght ();
      n = (Node) _graph.getNodeByPos (v);
      l += n.getWeight ();
      temp = new Path (min, l, v);

      _pi2 [v].add (temp);
      _c2 [v].remove (min);
    } else {
      temp = new Path (v);

      _pi2 [v].add (temp);
    }
  }

  /**
   * nextPath3 Method
   * @param v the node.
   * @param k the current position.
   */
  public void nextPath3 (int v, int k) {

    Integer integerVal;
    Node n;
    int intVal, kp;
    Path p, temp;

    if (v != this._source3) {
      if (k == 1) {
        n = (Node) _graph.getNodeByPos (v);
        this._c3 [v] = new LinkedList ();

        for (ListIterator i = n.getPrev ().listIterator (); i.hasNext (); ) {
          integerVal = (Integer) i.next ();
          intVal = integerVal.intValue ();
          p = (Path) _pi3 [intVal].getFirst ();
          temp = (Path) _pi3 [v].getFirst ();

          if (p != temp.getBack ()) {
            _c3 [v].add (p);
          }
        }
      }

      temp = (Path) _pi3 [v].get (k - 1);
      p = temp.getBack ();

      if (p != null) {
        intVal = p.getNode ();
        kp = _pi3 [intVal].indexOf (p);

        if ((kp + 1) == _pi3 [intVal].size ()) {
          nextPath3 (intVal, kp + 1);
        }

        temp = (Path) _pi3 [intVal].get (kp + 1);

        if (temp.getBack () != null) {
          _c3 [v].add (temp);
        }
      }
    }

    if (_c3 [v].size () != 0) {
      Path min = (Path) _c3 [v].getFirst ();
      float l;

      for (ListIterator i = _c3 [v].listIterator (); i.hasNext (); ) {
        temp = (Path) i.next ();

        if (temp.getLenght () < min.getLenght ()) {
          min = temp;
        }
      }

      l = min.getLenght ();
      n = (Node) _graph.getNodeByPos (v);
      l += n.getWeight ();
      temp = new Path (min, l, v);

      _pi3 [v].add (temp);
      _c3 [v].remove (min);
    } else {
      temp = new Path (v);

      _pi3 [v].add (temp);
    }
  }

  /**
   * nextPath4 Method
   * @param v the node.
   * @param k the current position.
   */
  public void nextPath4 (int v, int k) {

    Integer integerVal;
    Node n;
    int intVal, kp;
    Path p, temp;

    if (v != this._source4) {
      if (k == 1) {
        n = (Node) _graph.getNodeByPos (v);
        this._c4 [v] = new LinkedList ();

        for (ListIterator i = n.getPrev ().listIterator (); i.hasNext (); ) {
          integerVal = (Integer) i.next ();
          intVal = integerVal.intValue ();
          p = (Path) _pi4 [intVal].getFirst ();
          temp = (Path) _pi4 [v].getFirst ();

          if (p != temp.getBack ()) {
            _c4 [v].add (p);
          }
        }
      }

      temp = (Path) _pi4 [v].get (k - 1);
      p = temp.getBack ();

      if (p != null) {
        intVal = p.getNode ();
        kp = _pi4 [intVal].indexOf (p);

        if ((kp + 1) == _pi4 [intVal].size ()) {
          nextPath4 (intVal, kp + 1);
        }

        temp = (Path) _pi4 [intVal].get (kp + 1);

        if (temp.getBack () != null) {
          _c4 [v].add (temp);
        }
      }
    }

    if (_c4 [v].size () != 0) {
      Path min = (Path) _c4 [v].getFirst ();
      float l;

      for (ListIterator i = _c4 [v].listIterator (); i.hasNext (); ) {
        temp = (Path) i.next ();

        if (temp.getLenght () < min.getLenght ()) {
          min = temp;
        }
      }

      l = min.getLenght ();
      n = (Node) _graph.getNodeByPos (v);
      l += n.getWeight ();
      temp = new Path (min, l, v);

      _pi4 [v].add (temp);
      _c4 [v].remove (min);
    } else {
      temp = new Path (v);

      _pi4 [v].add (temp);
    }
  }

  /**
   * buildPath Method
   * @param target the destination position.
   * @param k the researched path.
   * @return a boolean
   */
  public boolean buildPath (int target, int k) {

    Node elem;
    LinkedList marked = new LinkedList ();
    int posit;
    int depht = 0;

    _currentWay = new LinkedList ();

    Path p;

    p = (Path) _pi [target].get (k);
    _currentDistance = p.getLenght ();

    if (p.getLenght () < _limit) {
      do {
        posit = p.getNode ();
        elem = (Node) _graph.getNodeByPos (posit);

        if (elem.marked ()) {
          removeMark (marked);

          return false;
        } else {
          elem.mark ();
          marked.add (elem);
          _currentWay.addFirst (elem.getId ());

          p = p.getBack ();

          depht++;
        }

        if (depht > _levelMax) {
          removeMark (marked);

          return false;
        }
      } while (p.getBack () != p);

      posit = p.getNode ();
      elem = (Node) _graph.getNodeByPos (posit);

      if (!elem.marked ()) {
        _currentWay.addFirst (elem.getId ());

        _currentDistance += elem.getWeight ();
      } else {
        removeMark (marked);

        return false;
      }

      updateSolution ();
      removeMark (marked);

      return true;
    }

    return false;
  }

  /**
   * buildPath2 Method
   * @param target the destination node.
   * @param k the researched path.
   * @return the status.
   */
  public boolean buildPath2 (int target, int k) {

    Node elem;
    LinkedList marked = new LinkedList ();
    int posit;
    int depht = 0;

    _currentWay = new LinkedList ();

    Path p;

    p = (Path) _pi2 [target].get (k);
    _currentDistance = p.getLenght ();

    if (p.getLenght () < _limit) {
      do {
        posit = p.getNode ();
        elem = (Node) _graph.getNodeByPos (posit);

        if (elem.marked ()) {
          removeMark (marked);

          return false;
        } else {
          elem.mark ();
          marked.add (elem);
          _currentWay.addFirst (elem.getId ());

          p = p.getBack ();

          depht++;
        }

        if (depht > _levelMax) {
          removeMark (marked);

          return false;
        }
      } while (p.getBack () != p);

      posit = p.getNode ();
      elem = (Node) _graph.getNodeByPos (posit);

      if (!elem.marked ()) {
        _currentWay.addFirst (elem.getId ());

        _currentDistance += elem.getWeight ();
      } else {
        removeMark (marked);

        return false;
      }

      updateSolution ();

      // System.out.println(k+"th path ACCEPTED");
      removeMark (marked);

      return true;
    }

    return false;
  }

  /**
   * buildPath3 Method
   * @param target the destination node.
   * @param k the researched path.
   * @return the status.
   */
  public boolean buildPath3 (int target, int k) {

    Node elem;
    LinkedList marked = new LinkedList ();
    int posit;
    int depht = 0;

    _currentWay = new LinkedList ();

    Path p;

    p = (Path) _pi3 [target].get (k);
    _currentDistance = p.getLenght ();

    if (p.getLenght () < _limit) {
      do {
        posit = p.getNode ();
        elem = (Node) _graph.getNodeByPos (posit);

        if (elem.marked ()) {
          removeMark (marked);

          return false;
        } else {
          elem.mark ();
          marked.add (elem);
          _currentWay.addFirst (elem.getId ());

          p = p.getBack ();

          depht++;
        }

        if (depht > _levelMax) {
          removeMark (marked);

          return false;
        }
      } while (p.getBack () != p);

      posit = p.getNode ();
      elem = (Node) _graph.getNodeByPos (posit);

      if (!elem.marked ()) {
        _currentWay.addFirst (elem.getId ());

        _currentDistance += elem.getWeight ();
      } else {
        removeMark (marked);

        return false;
      }

      updateSolution ();
      removeMark (marked);

      return true;
    }

    return false;
  }

  /**
   * buildPath4 Method
   * @param target the destination node.
   * @param k the researched path.
   * @return the status.
   * @return
   */
  public boolean buildPath4 (int target, int k) {

    Node elem;
    LinkedList marked = new LinkedList ();
    int posit;
    int depht = 0;

    _currentWay = new LinkedList ();

    Path p;

    p = (Path) _pi4 [target].get (k);
    _currentDistance = p.getLenght ();

    if (p.getLenght () < _limit) {
      do {
        posit = p.getNode ();
        elem = (Node) _graph.getNodeByPos (posit);

        if (elem.marked ()) {
          removeMark (marked);

          return false;
        } else {
          elem.mark ();
          marked.add (elem);
          _currentWay.addFirst (elem.getId ());

          p = p.getBack ();

          depht++;
        }

        if (depht > _levelMax) {
          removeMark (marked);

          return false;
        }
      } while (p.getBack () != p);

      posit = p.getNode ();
      elem = (Node) _graph.getNodeByPos (posit);

      if (!elem.marked ()) {
        _currentWay.addFirst (elem.getId ());

        _currentDistance += elem.getWeight ();
      } else {
        removeMark (marked);

        return false;
      }

      updateSolution ();
      removeMark (marked);

      return true;
    }

    return false;
  }

  /**
   * version of REA that uses Dijkstra with constraints
   * @param startNode A Node
   */
  public void rea2 (Node startNode) {

    // 
    assert startNode != null;

    // 
    System.out.println ("Enter rea2");

    boolean accept = true;
    int t = _graph.getCompoundPositionById (_endPointId);
    int n = _nbSolutions;
    long inTime;

    inTime = System.currentTimeMillis ();

    Path last;

    dijkstraWithConstraints (startNode);
    
    accept = buildPath (t, 0);

    if (!accept) {
      n++;
    }

    _c = new LinkedList [_pos];
    this._c [_source] = new LinkedList ();

    for (int k = 1; k < n; ++k) {
      nextPath (t, k);

      accept = buildPath (t, k);

      if (!accept) {
        n++;
      }

      last = (Path) _pi [t].get (k);

      if (last.getBack () == null) {
        k += n;
      }

      if (System.currentTimeMillis () - inTime >= _timeout) {
        _timeOver = true;
        k += n;
      }
    }
  }

  /* It's the version of REA that uses Dijkstra with constraints */

  /**
   * REA2 Method
   * @param startNode1 the first node.
   * @param startNode2 the second node.
   */
  public void rea2 (Node startNode1, Node startNode2) {

    // 
    assert startNode1 != null;

    assert startNode2 != null;

    // 
    boolean accept = true;
    LinkedList couple = (LinkedList) _graph.getReactionPositionsById (_endPointId);
    Integer endPosition1 = (Integer) couple.getFirst ();
    Integer endPosition2 = (Integer) couple.getLast ();
    int endPos1 = endPosition1.intValue ();
    int endPos2 = endPosition2.intValue ();
    int n = _nbSolutions;
    int k1 = 1;
    int k2 = 1;
    int k3 = 1;
    int k4 = 1;
    int min;
    float minVal;
    float niveau [] = new float [4];
    long inTime;

    inTime = System.currentTimeMillis ();

    Path last, p;

    last = null;
    n += 3;

    dijkstraWithConstraints (startNode1);

    accept = buildPath (endPos1, 0);

    if (!accept) {
      n++;
    }

    dijkstraWithConstraints2 (startNode2);

    accept = buildPath2 (endPos1, 0);

    if (!accept) {
      n++;
    }

    dijkstraWithConstraints3 (startNode1);

    accept = buildPath3 (endPos2, 0);

    if (!accept) {
      n++;
    }

    dijkstraWithConstraints4 (startNode2);

    accept = buildPath4 (endPos2, 0);

    if (!accept) {
      n++;
    }

    _c = new LinkedList [_pos];
    this._c [_source] = new LinkedList ();
    _c2 = new LinkedList [_pos];
    this._c2 [_source2] = new LinkedList ();
    _c3 = new LinkedList [_pos];
    this._c3 [_source3] = new LinkedList ();
    _c4 = new LinkedList [_pos];
    this._c4 [_source4] = new LinkedList ();

    while ((k1 + k2 + k3 + k4) < n) {
      p = (Path) _pi [endPos1].get (k1 - 1);
      niveau [0] = p.getLenght ();
      p = (Path) _pi2 [endPos1].get (k2 - 1);
      niveau [1] = p.getLenght ();
      p = (Path) _pi3 [endPos2].get (k3 - 1);
      niveau [2] = p.getLenght ();
      p = (Path) _pi4 [endPos2].get (k4 - 1);
      niveau [3] = p.getLenght ();
      minVal = niveau [0];
      min = 0;

      for (int i = 1; i < 4; i++) {
        if (niveau [i] < minVal) {
          minVal = niveau [i];
          min = i;
        }
      }

      switch (min) {

        case 0 :
          nextPath (endPos1, k1);

          accept = buildPath (endPos1, k1);
          last = (Path) _pi [endPos1].get (k1);

          k1++;
          break;

        case 1 :
          nextPath2 (endPos1, k2);

          accept = buildPath2 (endPos1, k2);
          last = (Path) _pi2 [endPos1].get (k2);

          k2++;
          break;

        case 2 :
          nextPath3 (endPos2, k3);

          accept = buildPath3 (endPos2, k3);
          last = (Path) _pi3 [endPos2].get (k3);

          k3++;
          break;

        case 3 :
          nextPath4 (endPos2, k4);

          accept = buildPath4 (endPos2, k4);
          last = (Path) _pi4 [endPos2].get (k4);

          k4++;
          break;
      }

      if (!accept) {
        n++;
      }

      if (last.getBack () == null) {
        k1 += n;
      }

      if (System.currentTimeMillis () - inTime >= _timeout) {
        _timeOver = true;
        k1 += n;
      }
    }

    // System.out.println("Total number: "+n);
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////
  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /**  */
  private Graph _graph;

  /**  */
  private String _startPointId;

  /**  */
  private String _endPointId;

  /**  */
  private int _maxLevel;

  /**  */
  private float _currentDistance;

  /**  */
  private float _maxDistance;

  /**  */
  private int _nbSolutions;

  /**  */
  private LinkedList _currentWay;

  /**  */
  private ArrayList _solutions;

  /** Field pos */
  private int _pos;

  /** Field limit */
  private float _limit = 1000;

  /** Field levelMax */
  private int _levelMax = 9;

  /** Field _startNode */
  private Node _startNode;

  /** Field timeOver */
  private boolean _timeOver;

  /** Field timeout */
  private long _timeout = 300000;

  /** Field_isInitialized */
  private boolean _isInitialized = false;

  /* This is the structures which are needed to do the REA method */

  /** Field source */
  private int _source;

  /** Field source2 */
  private int _source2;

  /** Field source3 */
  private int _source3;

  /** Field source4 */
  private int _source4;

  /** Field C[] */
  private LinkedList _c [];

  /** Field C2[] */
  private LinkedList _c2 [];

  /** Field C3[] */
  private LinkedList _c3 [];

  /** Field C4[] */
  private LinkedList _c4 [];

  /** Field PI[] */
  private LinkedList _pi [];

  /** Field PI2[] */
  private LinkedList _pi2 [];

  /** Field PI3[] */
  private LinkedList _pi3 [];

  /** Field PI4[] */
  private LinkedList _pi4 [];
/**
 * @return Returns the _timeout.
 */
public long getTimeout() {
	return _timeout;
}
/**
 * @param _timeout The _timeout to set.
 */
public void setTimeout(long _timeout) {
	this._timeout = _timeout;
}
}


/*--- Formatted in North Style ---*/
