package graph.weighted.weightedNodes;



import graph.weighted.Descriptions;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;


/**
 * Class: Result
 * Description: contains a result path and his lenght
 *
 * @author Fabian Couche
 * @version $Revision: 1.5 $
 */
public class Result
  implements Serializable,Comparator {

  /** Field path */
  private LinkedList _path;

  /** Field distance */
  private float _distance,_score;
  private Node sourceNode;
  private Node targetNode;

  /**
   * Result Constructor
   */
  public Result () {
    _path = new LinkedList ();
    _distance = 0;
  }

  /**
   * Result Constructor
   * @param l the list for the path.
   * @param d the distance.
   */
  public Result (List l, float s) {
    _path = new LinkedList (l);
    _score = s;
  }

  /**
   * getDistance Method
   * @return The distance of a result pathway.
   */
  public float getDistance () {
    return _distance;
    //return getScore();
  }

  /**
   * writeObject Method to serialize Result
   * @param out the stream.
   */
  protected void writeObject (ObjectOutputStream out) {

    Float d = new Float (_distance);

    try {
      out.writeObject (_path);
      out.writeObject (d);
    } catch (Exception e) {
      System.out.println ("Exception during serialization: " + e);
      System.exit (0);
    }
  }

  /**
   * readObject Method to serialize Result
   * @param in the stream.
   * @throws ClassNotFoundException an exception.
   * @throws IOException an exception.
   */
  protected void readObject (java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {

    Float d;

    _path = (LinkedList) in.readObject ();
    d = (Float) in.readObject ();
    _distance = d.floatValue ();
  }

  /**
   * textDisplay Method
   * Prints a pathway and his weight.
   * @param descript the descriptions of the elements.
   */
  public void textDisplay (Descriptions descript) {

    String ident;

    System.out.println ("Way:");

    for (ListIterator i = _path.listIterator (); i.hasNext (); ) {
      ident = (String) i.next ();

      System.out.print ("  ");
      System.out.println (descript.getName (ident));
    }

    System.out.println ();
    System.out.print ("Weight: ");
    System.out.println (_distance);
  }

  /**
   * textDisplay Method
   */
  public void textDisplay () {

    String ident;

    System.out.println ("Way:");

    for (ListIterator i = _path.listIterator (); i.hasNext (); ) {
      ident = (String) i.next ();

      System.out.print ("  ");
      System.out.println (ident);
    }

    System.out.println ();
    System.out.print ("Weight: ");
    System.out.println (_distance);
  }

  /**
   * Export the result in text format.
   * @return the string for the result.
   * @throws Exception an Exception.
   */
  public String textExport ()
    throws Exception {

    String line, temp, part;

    line = new String (_distance + "\t");

    for (ListIterator i = _path.listIterator (); i.hasNext (); ) {
      part = (String) i.next ();
      temp = line;
      line = temp.concat (part);
      temp = line;
      line = temp.concat ("\t");
    }

    return (line);
  }

  /**
   * Gets the path.
   * @return the solution path.
   */
  public LinkedList getPath () {
    return _path;
  }

/* (non-Javadoc)
 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
 */
public int compare(Object o1, Object o2) {
	// TODO Auto-generated method stub
	//if(!(o1 instanceof Result)|| !(o2 instanceof Result) ) throw  new Exception ("");
    float res1 = ((Result) o1).getDistance();
    float res2 = ((Result) o2).getDistance();
    
    if (res1 >= res2) return 1;
    else return -1;
}

 
/**
 * @return Returns the sourceNode.
 */
public Node getSourceNode() {
	return sourceNode;
}
/**
 * @param sourceNode The sourceNode to set.
 */
public void setSourceNode(Node sourceNode) {
	this.sourceNode = sourceNode;
}
/**
 * @return Returns the targetNode.
 */
public Node getTargetNode() {
	return targetNode;
}
/**
 * @param targetNode The targetNode to set.
 */
public void setTargetNode(Node targetNode) {
	this.targetNode = targetNode;
}
/**
 * @param distance The distance to set.
 */
public void setDistance(float distance) {
	_distance = distance;
}
/**
 * @return Returns the score.
 */
public float getScore() {
	return _score;
}
/**
 * @param score The score to set.
 */
public void setScore(float score) {
	_score = score;
}
}


/*--- Formatted in North Style ---*/
