package graph.weighted.weightedNodes;



import graph.weighted.Descriptions;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;


/**
 * Class: Solutions
 * Description: Serialization of the pathfinding solutions.
 *
 * @author Fabian Couche
 * @version $Revision: 1.4 $
 */
public class Solutions
  implements Serializable {

  /** Field pathways */
  private ArrayList _pathways;
  protected long searchTime = 0;

  /**
   * Solutions Constructor is the default constructor.
   */
  public Solutions () {
    _pathways = new ArrayList ();
  }

  /**
   * Solutions Constructor from a saved Solution
   * @param fileName The name of the file
   */
  public Solutions (String fileName) {

    Solutions temp = read (fileName);

    this._pathways = temp._pathways;
  }

  /**
   * writeObject Method must be done to serialize Solutions
   * @param out a stream.
   */
  protected void writeObject (ObjectOutputStream out) {

    try {
      out.writeObject (_pathways);
    } catch (Exception e) {
      System.out.println ("Exception during serialization: " + e);
      System.exit (0);
    }
  }

  /**
   * readObject Method must be done to serialize Solutions
   * @param in a stream.
   * @throws ClassNotFoundException an exception.
   * @throws IOException an exception.
   */
  protected void readObject (java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {

    _pathways = (ArrayList) in.readObject ();

    System.out.println ("pathways read\t" + _pathways.size ());
  }

  /**
   * setPathways Method
   * @param l the list of paths.
   */
  public void setPathways (List l) {
    _pathways.clear ();
    _pathways.addAll (l);
  }
  public void addPathways (List l) {
    _pathways.addAll (l);
  }
  
  

  /**
   * isEmpty Method tests if there is anything in the solutions.
   * @return true if it's empty.
   */
  public boolean isEmpty () {
    return _pathways.isEmpty ();
  }

  /**
   * Gets the size of the solutions set.
   * @return the current number of solutions.
   */
  public int getSize () {
    return _pathways.size ();
  }

  /**
   * void store(String fileName)  is a procedure to save the Solutions in a *  file.
   * @param fileName
   */

  /**
   * store Method saves the Solutions in a file.
   * @param fileName the name of the file.
   */
  public void store (String fileName) {

    Solutions sol = this;

    try {
      FileOutputStream fos = new FileOutputStream (fileName);
      ObjectOutputStream oos = new ObjectOutputStream (fos);

      oos.writeObject (sol);
      oos.flush ();
      oos.close ();
    } catch (Exception e) {
      System.out.println ("Exception during serialization: " + e);
      System.exit (0);
    }
  }

  /**
   * textExport Method
   * @param id the identifier.
   * @param fileName the file name.
   * @throws Exception an exception.
   */
  public void textExport (int id, String fileName)
    throws Exception {

    FileWriter file = new FileWriter (fileName, true);
    BufferedWriter buff = new BufferedWriter (file);
    String line;
    Result r;

    for (int p = 0; p < _pathways.size (); ++p) {
      line = new String (id + "\t" + p + "\t");
      r = (Result) _pathways.get (p);

      buff.write (line);

      line = r.textExport ();

      buff.write (line);
      buff.newLine ();
    }

    buff.close ();
  }

  /**
   * read Method reads the Solutions in a file.
   * @param fileName the file name.
   * @return the solutions from the file.
   */
  public Solutions read (String fileName) {

    Solutions back = new Solutions ();

    try {
      FileInputStream fis = new FileInputStream (fileName);
      ObjectInputStream ois = new ObjectInputStream (fis);

      back = (Solutions) ois.readObject ();

      ois.close ();
    } catch (Exception e) {
      System.out.println ("Exception during deserialization: " + e);
      System.exit (0);
    }

    return (back);
  }

  /**
   * textDisplay Method
   * @param desc the descriptions.
   */
  public void textDisplay (Descriptions desc) {

    Result r;

    System.out.println ("Found paths: ");
    System.out.println ();

    for (int i = 0; i < _pathways.size (); ++i) {
      System.out.print ("( ");
      System.out.print (i + 1);
      System.out.println (" )");

      r = (Result) _pathways.get (i);

      r.textDisplay (desc);
      System.out.println ();
    }
  }

  /**
   * textDisplay Method
   */
  public void textDisplay () {

    Result r;

    System.out.println ("Found paths: ");
    System.out.println ();

    for (int i = 0; i < _pathways.size (); ++i) {
      System.out.print ("( ");
      System.out.print (i + 1);
      System.out.println (" )");

      r = (Result) _pathways.get (i);  // r.textDisplay(desc);

      r.textDisplay ();
      System.out.println ();
    }
  }

  /**
   * Returns the list of paths.
   * @return the pathways.
   */
  public ArrayList getPathways () {
    return (_pathways);
  }
/**
 * @return Returns the searchTime.
 */
public long getSearchTime() {
	return searchTime;
}
/**
 * @param searchTime The searchTime to set.
 */
public void setSearchTime(long searchTime) {
	this.searchTime = searchTime;
}
}


/*--- Formatted in North Style ---*/
