package graph.weighted.weightedNodes;



import graph.weighted.GraphData;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import java.util.Set;


/**
 * Class: StatisticApp
 * Description: An application to do the statistics on pathfinding algorithms
 *
 * @author Fabian Couche
 * @version $Revision: 1.3 $
 */
public class StatisticApp {

  /** Field timeLimit */
  private static int _timeLimit = 300000;

  /**
   * StatisticApp Constructor
   */
  public StatisticApp () {
  }

  /**
   * main Method
   * @param args the arguments of the application.
   * @throws Exception the raised exception.
   */
  public static void main (String [] args)
    throws Exception {

    Solutions sol = new Solutions ();
    String graphDataFile = new String ("metabolic.gdt");
    GraphData gData = new GraphData (graphDataFile);
    Graph g = gData.getGraph ();
    Set s = g.keySetCompounds ();
    int taille = s.size ();
    Object c [] = s.toArray ();
    Random randCompound = new Random ();
    int i;
    int max = 100;
    int id = 900;
    boolean timeOut;
    long inTime = 0;
    long totalTime = 0;
    String sourceID, targetID;
    String line = null;

    for (int couple = 0; couple < max; couple++) {
      FileWriter file = new FileWriter ("tests.txt", true);
      BufferedWriter buff = new BufferedWriter (file);

      do {
        i = Math.abs (randCompound.nextInt () % taille);
        sourceID = (String) c [i];
        i = Math.abs (randCompound.nextInt () % taille);
        targetID = (String) c [i];

        sol.setPathways ((ArrayList) g.pathFinding (1, sourceID, targetID, 500, 8, 2));
      } while (sol.isEmpty ());

      for (int algo = 0; algo < 3; algo++) {
        for (int nbReact = 2; nbReact <= 8; nbReact *= 2) {
          for (float weightMax = 100; weightMax <= 500; weightMax *= 5) {
            timeOut = false;

            for (int nbSol = 1; nbSol <= 256; nbSol *= 4) {
              if (!timeOut) {
                inTime = System.currentTimeMillis ();

                sol.setPathways ((ArrayList) g.pathFinding (nbSol, sourceID, targetID, weightMax, nbReact, algo));

                totalTime = System.currentTimeMillis () - inTime;

                if (totalTime > _timeLimit) {
                  timeOut = true;
                } else if (!sol.isEmpty ()) {
                  sol.textExport (id, "paths.txt");
                }
              }

              if (timeOut) {
                totalTime = _timeLimit;
              }

              line = new String (id + "\t" + sourceID + "\t" + targetID + "\t" + algo + "\t" + nbReact + "\t"
                                 + weightMax + "\t" + nbSol + "\t" + sol.getSize () + "\t" + timeOut + "\t"
                                 + totalTime);

              try {
                buff.write (line);
                buff.newLine ();
              } catch (IOException e) {

                // nothing to say.
              }

              id++;
            }
          }
        }
      }

      buff.close ();
    }
  }
}


/*--- Formatted in North Style ---*/
