/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



/**
 * Class of support to implement the IArcProvider interface.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: ArcProviderSupport.java,v 1.2 2004/11/18 15:26:38 simon Exp $
 */
public class ArcProviderSupport
  implements IArcProvider {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected ArcProviderSupport () {
  }

  /**
   * Creates a new instance.
   * @param arc the held arc.
   * @return the new instance.
   */
  public static ArcProviderSupport newArcProviderSupport (IArc arc) {

    ArcProviderSupport result = new ArcProviderSupport ();

    result.init (arc);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Init method.
   * @param arc the held arc.
   */
  protected void init (IArc arc) {

    // 
    assert arc != null;

    // 
    this.setArc (arc);

    // 
    assert this.getArc () == arc;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.IArcProvider#getArc()
   */
  public IArc getArc () {

    // 
    assert this._arc != null;

    // 
    return this._arc;
  }

  /**
   * The arc to set.
   * @param arc the given arc.
   */
  protected void setArc (IArc arc) {

    // 
    assert arc != null;

    // 
    this._arc = arc;

    // 
    assert this.getArc () == arc;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Comparison
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals (Object obj) {

    boolean result = false;

    if (obj instanceof IArcProvider) {
      this.getArc ().equals (((IArcProvider) obj).getArc ());
    }

    return result;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode () {
    return this.getArc ().hashCode ();
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** The held arc */
  IArc _arc = null;
}


/*--- Formatted in North Style ---*/
