/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



/**
 * Class of support to implement the ICharacterized interface.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: CharacterizationSupport.java,v 1.3 2004/10/13 07:35:10 fabian Exp $
 */
public class CharacterizationSupport {

  /**
   * Tests if the given attribute's key is either contained in the characterized object or in the data repository.
   * @param characterized the characterized object.
   * @param data the data repository.
   * @param key the attribute's key.
   * @return true if the given attribute's key is either contained in the characterized object or in the data repository.
   */
  public static boolean containsAttribute (ICharacterized characterized, IData data, String key) {

    // 
    assert characterized != null;

    assert data != null;

    assert key != null;

    // 
    boolean result = characterized.containsAttribute (key) || data.containsAttribute (characterized, key);

    return result;
  }

  /**
   * Returns the attribute associated to the given key in the specified data repository if it's present otherwise in the characterized object.
   * @param characterized the characterized object.
   * @param data the data repository.
   * @param key the attribute's key.
   * @return the attribute associated to the given key in the specified data repository if it's present otherwise in the characterized object.
   */
  public static Object getValue (ICharacterized characterized, IData data, String key) {

    // 
    assert characterized != null;

    assert data != null;

    assert key != null;

    if (!containsAttribute (characterized, data, key)) {
      throw new IllegalArgumentException (
        "Neither the specified characterized object nor the data repository contains the specified attribute.");
    }

    // 
    Object result = null;

    if (data.containsAttribute (characterized, key)) {
      result = data.getValue (characterized, key);
    } else if (characterized.containsAttribute (key)) {
      result = characterized.getValue (key);
    } else {

      // COULD NEVER HAPPEN
      throw new IllegalStateException ("You just did something impossible.");
    }

    // 
    assert result != null;

    // 
    return result;
  }

  // //////////////////////////////////////////////////////////
  // 
  // Element change
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Puts the given value in the specified data repository by associating it to the characterized object and to the key .
   * @param characterized the characterized object.
   * @param data the data repository.
   * @param key the attribute's key.
   * @param value the value.
   */
  public static void putAttribute (ICharacterized characterized, IEditableData data, String key, Object value) {

    // 
    assert characterized != null;

    assert data != null;

    assert key != null;

    assert value != null;

    // 
    data.putAttribute (characterized, key, value);

    // 
    assert data.getValue (characterized, key) == value;

    // 
  }
}


/*--- Formatted in North Style ---*/
