/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



/**
 * Class of support to implement the IGraphProvider interface.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: GraphProviderSupport.java,v 1.1 2004/12/02 15:07:41 frederic Exp $
 */
public class GraphProviderSupport
  implements IGraphProvider {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected GraphProviderSupport () {
  }

  /**
   * Creates a new instance.
   * @param graph the held graph.
   * @return the new instance.
   */
  public static GraphProviderSupport newGraphProviderSupport (IGraph graph) {

    GraphProviderSupport result = new GraphProviderSupport ();

    result.init (graph);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Init method.
   * @param graph the held graph.
   */
  protected void init (IGraph graph) {

    // 
    assert graph != null;

    // 
    this.setGraph (graph);

    // 
    assert this.getGraph () == graph;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.IGraphProvider#getGraph()
   */
  public IGraph getGraph () {

    // 
    assert this._graph != null;

    // 
    return this._graph;
  }

  /**
   * The graph to set.
   * @param graph the given graph.
   */
  protected void setGraph (IGraph graph) {

    // 
    assert graph != null;

    // 
    this._graph = graph;

    // 
    assert this.getGraph () == graph;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Comparison
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals (Object obj) {

    boolean result = false;

    if (obj instanceof IGraphProvider) {
      result = this.getGraph ().equals (((IGraphProvider) obj).getGraph ());
    }

    return result;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode () {
    return this.getGraph ().hashCode ();
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** The held graph */
  IGraph _graph = null;
}


/*--- Formatted in North Style ---*/
