/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



import java.util.Set;


/**
 * Characterized Interface which defines an object with a type and attributes.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: ICharacterized.java,v 1.8 2004/10/13 07:35:10 fabian Exp $
 */
public interface ICharacterized {

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns a String containing the identifier of the object.
   * @return the identifier of the object.
   */
  String getIdentifier ();

  /**
   * Returns a String containing the label of the object.
   * @return the label of the object.
   */
  String getLabel ();

  /**
   * Returns a String containing the type of the object.
   * @return the type of the object.
   */
  String getType ();

  /**
   * Returns the attribute associated with the specified key.
   * @param key the specified attribute key.
   * @return the attribute associated with the specified key.
   */
  Object getValue (String key);

  /**
   * Returns the set of the attributes' keys.
   * @return the set of the attributes' keys.
   */
  Set getKeys ();

  /**
   * Returns the attribute associated to the given key in the specified data repository if it's present otherwise in the characterized object.
   * @param data the data repository.
   * @param key the attribute's key.
   * @return the attribute associated to the given key in the specified data repository if it's present otherwise in the characterized object.
   */
  Object getValue (IData data, String key);

  // //////////////////////////////////////////////////////////
  // 
  // Status report
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns true if the identifier has been specified.
   * @return true if the identifier has been specified.
   */
  boolean hasIdentifier ();

  /**
   * Returns true if the label has been specified.
   * @return true if the label has been specified.
   */
  boolean hasLabel ();

  /**
   * Returns true if the type has been specified.
   * @return true if the type has been specified.
   */
  boolean hasType ();

  /**
   * Returns true if the attribute associated to the specified key is present in the object.
   * @param key the given key.
   * @return true if the attribute associated to the specified key is present in the object.
   */
  boolean containsAttribute (String key);

  /**
   * Tests if the given attribute's key is either contained in the characterized object or in the data repository.
   * @param data the data repository.
   * @param key the attribute's key.
   * @return true if the given attribute's key is either contained in the characterized object or in the data repository.
   */
  boolean containsAttribute (IData data, String key);

  // //////////////////////////////////////////////////////////
  // 
  // Element change
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Puts the given value in the specified data repository by associating it to the characterized object and to the key .
   * @param data the data repository.
   * @param key the attribute's key.
   * @param value the value.
   */
  void putAttribute (IEditableData data, String key, Object value);

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** the identifier key */
  public static final String IDENTIFIER_KEY = "identifier";

  /** the label key */
  public static final String LABEL_KEY = "label";
}


/*--- Formatted in North Style ---*/
