/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;

import java.util.Set;


/**
 * Graph Interface which is a minimum adapter.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: IGraph.java,v 1.6 2004/12/02 15:08:00 frederic Exp $
 */
public interface IGraph
  extends ICharacterized {

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns an unmodifiable set of INode.
   * @return the unmodifiable set of INode.
   */
  Set getNodes ();

  /**
   * Returns an unmodifiable set of IArc.
   * @return the unmodifiable set of IArc.
   */
  Set getArcs ();

  // //////////////////////////////////////////////////////////
  // 
  // JavaBeans
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Adds a PropertyChangeListener on the IGraph.
   * For an easy implementation @see java.beans.PropertyChangeSupport#addPropertyChangeListener(java.beans.PropertyChangeListener)
   * @param listener the listener to be added.
   */
  void addPropertyChangeListener (PropertyChangeListener listener);

  /**
   * Adds a PropertyChangeListener on the IGraph.
   * For an easy implementation @see java.beans.PropertyChangeSupport#addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
   * @param propertyName the property name.
   * @param listener the listener to be added.
   */
  void addPropertyChangeListener (String propertyName, PropertyChangeListener listener);

  /**
   * Removes a PropertyChangeListener from the IGraph.
   * For an easy implementation @see java.beans.PropertyChangeSupport#removePropertyChangeListener(java.beans.PropertyChangeListener)
   * @param listener the listener to be removed.
   */
  void removePropertyChangeListener (PropertyChangeListener listener);

  /**
   * Removes a PropertyChangeListener from the IGraph.
   * For an easy implementation @see java.beans.PropertyChangeSupport#removePropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
   * @param propertyName the property name.
   * @param listener the listener to be removed.
   */
  void removePropertyChangeListener (String propertyName, PropertyChangeListener listener);

  /**
   * Reports a bound property update to any registered listeners.
   * For an easy implementation @see java.beans.PropertyChangeSupport#firePropertyChange(java.lang.String, java.lang.Object, java.lang.Object)
   * @param propertyName the property name.
   * @param oldValue the old value.
   * @param newValue the new value.
   */
  void firePropertyChange (String propertyName, Object oldValue, Object newValue);

  /**
   * Adds a VetoableChangeListener on the IGraph.
   * For an easy implementation @see java.beans.VetoableChangeSupport#addVetoableChangeListener(java.beans.VetoableChangeListener)
   * @param listener the listener to be added.
   */
  void addVetoableChangeListener (VetoableChangeListener listener);

  /**
   * Adds a VetoableChangeListener on the IGraph.
   * For an easy implementation @see java.beans.VetoableChangeSupport#addVetoableChangeListener(java.lang.String, java.beans.VetoableChangeListener)
   * @param propertyName the property name.
   * @param listener the listener to be added.
   */
  void addVetoableChangeListener (String propertyName, VetoableChangeListener listener);

  /**
   * Removes a VetoableChangeListener from the IGraph.
   * For an easy implementation @see java.beans.VetoableChangeSupport#removeVetoableChangeListener(java.beans.VetoableChangeListener)
   * @param listener the listener to be removed.
   */
  void removeVetoableChangeListener (VetoableChangeListener listener);

  /**
   * Removes a VetoableChangeListener from the IGraph.
   * For an easy implementation @see java.beans.VetoableChangeSupport#removeVetoableChangeListener(java.lang.String, java.beans.VetoableChangeListener)
   * @param propertyName the property name.
   * @param listener the listener to be removed.
   */
  void removeVetoableChangeListener (String propertyName, VetoableChangeListener listener);

  /**
   * Reports a vetoable property update to any registered listeners.
   * For an easy implementation @see java.beans.VetoableChangeSupport#fireVetoableChange(java.lang.String, java.lang.Object, java.lang.Object)
   * @param propertyName the property name.
   * @param oldValue the old value.
   * @param newValue the new value.
   * @throws PropertyVetoException A PropertyVetoException is thrown when a proposed change to a  property represents an unacceptable value.
   */
  void fireVetoableChange (String propertyName, Object oldValue, Object newValue)
    throws PropertyVetoException;

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** Graph property name */
  public static final String GRAPH_PROPERTY = "graph";

  /** Node property name */
  public static final String NODE_PROPERTY = "node";

  /** Arc property name */
  public static final String ARC_PROPERTY = "arc";
}


/*--- Formatted in North Style ---*/
