/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core;



/**
 * Class of support to implement the INodeProvider interface.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: NodeProviderSupport.java,v 1.2 2004/11/18 15:26:38 simon Exp $
 */
public class NodeProviderSupport
  implements INodeProvider {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected NodeProviderSupport () {
  }

  /**
   * Creates a new instance.
   * @param node the held node.
   * @return the new instance.
   */
  public static NodeProviderSupport newNodeProviderSupport (INode node) {

    NodeProviderSupport result = new NodeProviderSupport ();

    result.init (node);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Init method.
   * @param node the held node.
   */
  protected void init (INode node) {

    // 
    assert node != null;

    // 
    this.setNode (node);

    // 
    assert this.getNode () == node;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.INodeProvider#getNode()
   */
  public INode getNode () {

    // 
    assert this._node != null;

    // 
    return this._node;
  }

  /**
   * The node to set.
   * @param node the given node.
   */
  protected void setNode (INode node) {

    // 
    assert node != null;

    // 
    this._node = node;

    // 
    assert this.getNode () == node;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Comparison
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals (Object obj) {

    boolean result = false;

    if (obj instanceof INodeProvider) {
      result = this.getNode ().equals (((INodeProvider) obj).getNode ());
    }

    return result;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  public int hashCode () {
    return this.getNode ().hashCode ();
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** The held node */
  INode _node = null;
}


/*--- Formatted in North Style ---*/
