/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core.basic;



import java.util.HashMap;

import north.graph.core.ICharacterized;
import north.graph.core.IEditableData;


/**
 * BasicCharacterized abstract class which is the basic implementation of the ICharacterized.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: BasicData.java,v 1.6 2004/10/13 07:35:10 fabian Exp $
 */
public class BasicData
  implements IEditableData {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected BasicData () {
  }

  /**
   * Creates a new instance.
   * @return the new instance.
   */
  public static BasicData newBasicData () {

    BasicData result = new BasicData ();

    result.init ();

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Init method.
   */
  protected void init () {
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns the data map.
   * @return the data map.
   */
  protected HashMap getDataMap () {

    // 
    assert this._dataMap != null;

    // 
    return this._dataMap;
  }

  /**
   * The data to set.
   * @param dataMap the given data.
   */
  protected void setDataMap (HashMap dataMap) {

    // 
    assert dataMap != null;

    // 
    this._dataMap = dataMap;

    // 
    assert this.getDataMap () == dataMap;

    // 
  }

  /**
   * @see north.graph.core.IData#containsAttribute(north.graph.core.ICharacterized, java.lang.String)
   */
  public boolean containsAttribute (ICharacterized characterized, String key) {

    // 
    assert characterized != null;

    assert key != null;

    // 
    boolean result = false;
    HashMap dataMap = this.getDataMap ();

    if (dataMap.containsKey (characterized)) {
      HashMap attributeMap = (HashMap) dataMap.get (characterized);

      result = attributeMap.containsKey (key);
    }

    return result;
  }

  /**
   * @see north.graph.core.IData#getValue(north.graph.core.ICharacterized, java.lang.String)
   */
  public Object getValue (ICharacterized characterized, String key) {

    // 
    assert characterized != null;

    assert key != null;

    assert this.containsAttribute (characterized, key);

    // 
    Object result = ((HashMap) this.getDataMap ().get (characterized)).get (key);

    // 
    assert result != null;

    // 
    return result;
  }

  // //////////////////////////////////////////////////////////
  // 
  // Element change
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.IEditableData#putAttribute(north.graph.core.ICharacterized, java.lang.String, java.lang.Object)
   */
  public void putAttribute (ICharacterized characterized, String key, Object value) {

    // 
    assert characterized != null;

    assert key != null;

    assert value != null;

    // 
    HashMap dataMap = this.getDataMap ();

    if (!dataMap.containsKey (characterized)) {
      dataMap.put (characterized, new HashMap ());
    }

    HashMap attributeMap = (HashMap) dataMap.get (characterized);

    attributeMap.put (key, value);

    // 
    assert attributeMap.get (key) != null;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** the attributes map of the characterized object */
  private HashMap _dataMap = new HashMap ();
}


/*--- Formatted in North Style ---*/
