/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core.basic;



import java.util.Calendar;
import java.util.Iterator;
import java.util.Set;

import junit.framework.TestCase;


/**
 * BasicGraphTest class.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: BasicGraphTest.java,v 1.4 2004/11/16 15:49:46 frederic Exp $
 */
public class BasicGraphTest
  extends TestCase {

  // //////////////////////////////////////////////////////////
  // 
  // Basic operations
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Tests getInArcs() method.
   */
  public void testGetInArcs () {

    // set the predecessors numbers.
    int predecessorsNumber = 5;

    // Build the test graph.
    BasicGraph graph = BasicGraph.newBasicGraph ();

    // Build the central node and add it to the graph.
    BasicNode node = BasicNode.newBasicNode (graph, new Long (Calendar.getInstance ().getTimeInMillis ()).toString (),
                                             "Test node", this.getClass ().getName ());

    graph.addNode (node);

    // Build the predecessors and the associated arcs.
    for (int i = 0; i < predecessorsNumber; i++) {
      BasicNode n = BasicNode.newBasicNode (graph, "prevNode" + i, "Test node" + i, this.getClass ().getName ());

      graph.addNode (n);

      BasicArc arc = BasicArc.newBasicArc (n, node);

      graph.addArc (arc);
    }

    // Test the method
    Set inArcs = graph.getInArcs (node);

    assertEquals (predecessorsNumber, inArcs.size ());

    for (Iterator iter = inArcs.iterator (); iter.hasNext (); ) {
      BasicArc arc = (BasicArc) iter.next ();
      String tailNodeId = ((BasicNode) arc.getTail ()).getIdentifier ();

      assertTrue (tailNodeId.indexOf ("prevNode") != -1);

      String headNodeId = ((BasicNode) arc.getHead ()).getIdentifier ();

      assertEquals (node.getIdentifier (), headNodeId);
    }
  }

  /**
   * Tests getOutArcs() method.
   */
  public void testGetOutArcs () {

    // set the successors numbers.
    int successorsNumber = 6;

    // Build the test graph.
    BasicGraph graph = BasicGraph.newBasicGraph ();

    // Build the central node and add it to the graph.
    BasicNode node = BasicNode.newBasicNode (graph, new Long (Calendar.getInstance ().getTimeInMillis ()).toString (),
                                             "Test node", this.getClass ().getName ());

    graph.addNode (node);

    // Build the successors and the associated arcs.
    for (int i = 0; i < successorsNumber; i++) {
      BasicNode n = BasicNode.newBasicNode (graph, "nextNode" + i, "Test node" + i, this.getClass ().getName ());

      graph.addNode (n);

      BasicArc arc = BasicArc.newBasicArc (node, n);

      graph.addArc (arc);
    }

    // Test the method
    Set outArcs = graph.getOutArcs (node);

    assertEquals (successorsNumber, outArcs.size ());

    for (Iterator iter = outArcs.iterator (); iter.hasNext (); ) {
      BasicArc arc = (BasicArc) iter.next ();
      String tailNodeId = ((BasicNode) arc.getTail ()).getIdentifier ();

      assertEquals (node.getIdentifier (), tailNodeId);

      String headNodeId = ((BasicNode) arc.getHead ()).getIdentifier ();

      assertTrue (headNodeId.indexOf ("nextNode") != -1);
    }
  }
}


/*--- Formatted in North Style ---*/
