/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core.basic;



import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.beans.VetoableChangeSupport;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import north.graph.core.ICharacterized;
import north.graph.core.IDOM;
import north.graph.core.IExportableArc;
import north.graph.core.IExportableNode;
import north.graph.core.IExportableSubgraph;
import north.graph.core.ISubdividedGraph;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 * Subgraph class.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: BasicSubgraph.java,v 1.18 2004/11/15 13:02:33 fabian Exp $
 */
public class BasicSubgraph
  extends BasicCharacterized
  implements IExportableSubgraph {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected BasicSubgraph () {
  }

  /**
   * Creates a new instance.
   * @param parent the graph parent.
   * @return the new instance.
   */
  public static BasicSubgraph newBasicSubgraph (BasicGraph parent) {

    // 
    assert parent != null;

    // 
    BasicSubgraph result = new BasicSubgraph ();

    result.init (parent);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Creates a new instance.
   * @param parent the graph parent.
   * @param identifier the identifier of the subgraph.
   * @param label the label of the subgraph.
   * @param type the type of the subgraph.
   * @return the new instance.
   */
  public static BasicSubgraph newBasicSubgraph (BasicGraph parent, String identifier, String label, String type) {

    // 
    assert parent != null;

    assert identifier != null;

    assert label != null;

    assert type != null;

    // 
    BasicSubgraph result = new BasicSubgraph ();

    result.init (parent, identifier, label, type);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Init method.
   * @param parent the graph parent.
   */
  protected void init (BasicGraph parent) {

    // 
    assert parent != null;

    // 
    this.setParent (parent);

    // 
    assert this.getParent () == parent;

    // 
  }

  /**
   * Init method.
   * @param parent the graph parent.
   * @param identifier the identifier of the subgraph.
   * @param label the label of the subgraph.
   * @param type the type of the subgraph.
   */
  protected void init (BasicGraph parent, String identifier, String label, String type) {

    // 
    assert parent != null;

    assert identifier != null;

    assert label != null;

    assert type != null;

    // 
    this.init (parent);
    super.init (identifier, label, type);

    // 
    assert this.getParent () == parent;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns the parent BasicGraph.
   * @return the parent BasicGraph.
   */
  protected BasicGraph getParent () {

    // 
    assert this._parent != null;

    // 
    return this._parent;
  }

  /**
   * The parent to set.
   * @param parent the given parent.
   */
  protected void setParent (BasicGraph parent) {

    // 
    assert parent != null;

    // 
    this._parent = parent;

    // 
    assert this.getParent () == parent;

    // 
  }

  /**
   * Gets the nodes map.
   * @return the nodes map.
   */
  protected HashMap getNodeMap () {

    // 
    assert this._nodeMap != null;

    // 
    return this._nodeMap;
  }

  /**
   * The nodes to set.
   * @param nodeMap the nodes set.
   */
  protected void setNodeMap (HashMap nodeMap) {

    // 
    assert nodeMap != null;

    // 
    this._nodeMap = nodeMap;

    // 
    assert this.getNodeMap () == nodeMap;

    // 
  }

  /**
   * Returns the arcs.
   * @return the arcs.
   */
  protected HashSet getArcSet () {

    // 
    assert this._arcSet != null;

    // 
    return this._arcSet;
  }

  /**
   * The arcs to set.
   * @param arcSet the arcs set.
   */
  protected void setArcSet (HashSet arcSet) {

    // 
    assert arcSet != null;

    // 
    this._arcSet = arcSet;

    // 
    assert this.getArcSet () == arcSet;

    // 
  }

  /**
   * Tells if the given node is present in the current graph.
   * @param node the tested node.
   * @return true if the given node is present in the current graph.
   */
  public boolean hasNode (BasicNode node) {

    // 
    assert node != null;

    // 
    boolean result = this.getNodeMap ().containsValue (node);

    return result;
  }

  /**
   * Tells if the given node identifier is present in the current graph.
   * @param nodeId the tested identifier.
   * @return true if the given identifier is present in the current graph.
   */
  public boolean hasNode (String nodeId) {

    // 
    assert nodeId != null;

    // 
    boolean result = this.getNodeMap ().containsKey (nodeId);

    return result;
  }

  /**
   * Tells if the given arc is present in the current graph.
   * @param arc the tested arc.
   * @return true if the given arc is present in the current graph.
   */
  public boolean hasArc (BasicArc arc) {

    // 
    assert arc != null;

    // 
    boolean result = this.getArcSet ().contains (arc);

    return result;
  }

  /**
   * @see north.graph.core.ISubgraph#getParentGraph()
   */
  public ISubdividedGraph getParentGraph () {

    ISubdividedGraph result = this.getParent ();

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * @see north.graph.core.IGraph#getNodes()
   */
  public Set getNodes () {
    return Collections.unmodifiableSet (new HashSet (this.getNodeMap ().values ()));
  }

  /**
   * @see north.graph.core.IGraph#getArcs()
   */
  public Set getArcs () {
    return Collections.unmodifiableSet (this.getArcSet ());
  }

  /**
   * Adds a node in the graph.
   * @param node the node to be added.
   */
  public void addNode (BasicNode node) {

    // 
    assert node != null;

    assert !this.hasNode (node.getIdentifier ());

    // 
    if (!this.getParent ().hasNode (node)) {
      throw new IllegalArgumentException ("A node of a subgraph must be contained in its parent graph!");
    }

    this.getNodeMap ().put (node.getIdentifier (), node);

    // 
    assert this.hasNode (node.getIdentifier ());

    // 
  }

  /**
   * Returns the node corresponding to the given identifier.
   * @param nodeId a node identifier.
   * @return the found node.
   */
  public BasicNode getNode (String nodeId) {

    // 
    assert nodeId != null;

    assert this.hasNode (nodeId);

    // 
    BasicNode result = (BasicNode) this.getNodeMap ().get (nodeId);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Adds an arc in the graph.
   * @param arc the arc to be added.
   */
  public void addArc (BasicArc arc) {

    // 
    assert arc != null;

    assert !this.hasArc (arc);

    // 
    if (!this.getParent ().hasArc (arc)) {
      throw new IllegalArgumentException ("A arc of a subgraph must be contained in its parent graph!");
    }

    this.getArcSet ().add (arc);

    // 
    assert this.hasArc (arc);

    assert this.hasArc (arc);

    // 
  }

  /**
   * @see north.graph.core.ICharacterized#hasIdentifier()
   */
  public boolean hasIdentifier () {
    return this.containsAttribute (ICharacterized.IDENTIFIER_KEY);
  }

  /**
   * @see north.graph.core.ICharacterized#getIdentifier()
   */
  public String getIdentifier () {

    String result = (String) this.getValue (ICharacterized.IDENTIFIER_KEY);

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * Sets the identifier.
   * @param id the identifier value.
   */
  public void setIdentifier (String id) {

    // 
    assert id != null;

    // 
    this.putAttribute (ICharacterized.IDENTIFIER_KEY, id);

    // 
    assert this.getIdentifier () == id;

    // 
  }

  /**
   * @see north.graph.core.ICharacterized#getKeys()
   */
  public Set getKeys () {

    Set result = super.getKeys ();

    // 
    // return the set of keys without the "identifier" key.
    // 
    if (result.contains (ICharacterized.IDENTIFIER_KEY)) {
      result.remove (ICharacterized.IDENTIFIER_KEY);
    }

    // 
    assert result != null;

    // 
    return result;
  }

  /**
   * @see north.graph.core.basic.BasicCharacterized#getDOMTagName()
   */
  protected String getDOMTagName () {
    return IDOM.SUBGRAPH;
  }

  /**
   * Returns the propertyChangeSupport.
   * @return the propertyChangeSupport.
   */
  protected PropertyChangeSupport getPropertyChangeSupport () {

    // 
    assert this._propertyChangeSupport != null;

    // 
    return this._propertyChangeSupport;
  }

  /**
   * Returns the vetoableChangeSupport.
   * @return the vetoableChangeSupport.
   */
  protected VetoableChangeSupport getVetoableChangeSupport () {

    // 
    assert this._vetoableChangeSupport != null;

    // 
    return this._vetoableChangeSupport;
  }

  // //////////////////////////////////////////////////////////
  // 
  // JavaBeans
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.IGraph#addPropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void addPropertyChangeListener (PropertyChangeListener listener) {

    // 
    assert listener != null;

    // 
    this.getPropertyChangeSupport ().addPropertyChangeListener (listener);
  }

  /**
   * @see north.graph.core.IGraph#addPropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
   */
  public void addPropertyChangeListener (String propertyName, PropertyChangeListener listener) {

    // 
    assert propertyName != null;

    assert listener != null;

    // 
    this.getPropertyChangeSupport ().addPropertyChangeListener (propertyName, listener);
  }

  /**
   * @see north.graph.core.IGraph#removePropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void removePropertyChangeListener (PropertyChangeListener listener) {

    // 
    assert listener != null;

    // 
    this.getPropertyChangeSupport ().removePropertyChangeListener (listener);
  }

  /**
   * @see north.graph.core.IGraph#removePropertyChangeListener(java.lang.String, java.beans.PropertyChangeListener)
   */
  public void removePropertyChangeListener (String propertyName, PropertyChangeListener listener) {

    // 
    assert propertyName != null;

    assert listener != null;

    // 
    this.getPropertyChangeSupport ().removePropertyChangeListener (propertyName, listener);
  }

  /**
   * @see north.graph.core.IGraph#firePropertyChange(java.lang.String, java.lang.Object, java.lang.Object)
   */
  public void firePropertyChange (String propertyName, Object oldValue, Object newValue) {

    // 
    assert propertyName != null;

    // 
    this.getPropertyChangeSupport ().firePropertyChange (propertyName, oldValue, newValue);
  }

  /**
   * @see north.graph.core.IGraph#addVetoableChangeListener(java.beans.VetoableChangeListener)
   */
  public void addVetoableChangeListener (VetoableChangeListener listener) {

    // 
    assert listener != null;

    // 
    this.getVetoableChangeSupport ().addVetoableChangeListener (listener);
  }

  /**
   * @see north.graph.core.IGraph#addVetoableChangeListener(java.lang.String, java.beans.VetoableChangeListener)
   */
  public void addVetoableChangeListener (String propertyName, VetoableChangeListener listener) {

    // 
    assert propertyName != null;

    assert listener != null;

    // 
    this.getVetoableChangeSupport ().addVetoableChangeListener (propertyName, listener);
  }

  /**
   * @see north.graph.core.IGraph#removeVetoableChangeListener(java.beans.VetoableChangeListener)
   */
  public void removeVetoableChangeListener (VetoableChangeListener listener) {

    // 
    assert listener != null;

    // 
    this.getVetoableChangeSupport ().removeVetoableChangeListener (listener);
  }

  /**
   * @see north.graph.core.IGraph#removeVetoableChangeListener(java.lang.String, java.beans.VetoableChangeListener)
   */
  public void removeVetoableChangeListener (String propertyName, VetoableChangeListener listener) {

    // 
    assert propertyName != null;

    assert listener != null;

    // 
    this.getVetoableChangeSupport ().removeVetoableChangeListener (propertyName, listener);
  }

  /**
   * @throws PropertyVetoException
   * @see north.graph.core.IGraph#fireVetoableChange(java.lang.String, java.lang.Object, java.lang.Object)
   */
  public void fireVetoableChange (String propertyName, Object oldValue, Object newValue)
    throws PropertyVetoException {

    // 
    assert propertyName != null;

    // 
    this.getVetoableChangeSupport ().fireVetoableChange (propertyName, oldValue, newValue);
  }

  // //////////////////////////////////////////////////////////
  // 
  // Output
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.core.IExportable#toElement(org.w3c.dom.Document)
   */
  public Element toElement (Document document) {

    // 
    assert document != null;

    // 
    // 
    // Create the DOM element
    // 
    Element result = super.toElement (document);

    result.setAttribute (IDOM.ID, this.getIdentifier ());

    if (this.hasType ()) {
      result.setAttribute (IDOM.TYPE, this.getType ());
    }

    // Nodes
    for (Iterator iter = this.getNodeMap ().values ().iterator (); iter.hasNext (); ) {
      IExportableNode node = (IExportableNode) iter.next ();
      Element sgNodeElt = document.createElement (IDOM.SG_NODE);

      result.appendChild (sgNodeElt);
      sgNodeElt.setAttribute (IDOM.REF, node.getIdentifier ());
    }

    // Arcs
    for (Iterator iter = this.getArcSet ().iterator (); iter.hasNext (); ) {
      IExportableArc arc = (IExportableArc) iter.next ();
      String tailId = ((IExportableNode) arc.getTail ()).getIdentifier ();
      String headId = ((IExportableNode) arc.getHead ()).getIdentifier ();
      Element sgArcElt = document.createElement (IDOM.SG_ARC);

      result.appendChild (sgArcElt);
      sgArcElt.setAttribute (IDOM.REF, tailId + "-" + headId);
    }

    // 
    assert result != null;

    // 
    return result;
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** the parent graph */
  private BasicGraph _parent;

  /** the nodes map */
  private HashMap _nodeMap = new HashMap ();

  /** the arcs set */
  private HashSet _arcSet = new HashSet ();

  /** The support object for the JavaBeans interface. */
  private PropertyChangeSupport _propertyChangeSupport = new PropertyChangeSupport (this);

  /** The vetoable support object for the JavaBeans interface. */
  private VetoableChangeSupport _vetoableChangeSupport = new VetoableChangeSupport (this);
}


/*--- Formatted in North Style ---*/
