/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core.builder;



import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import north.graph.antlr.IDotGraphBuilder;
import north.graph.core.basic.BasicArc;
import north.graph.core.basic.BasicGraph;
import north.graph.core.basic.BasicNode;
import north.graph.core.basic.BasicSubgraph;


/**
 * BasicGraphBuilder class.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: DotGraphBuilder.java,v 1.10 2004/11/16 15:50:44 frederic Exp $
 */
public class DotGraphBuilder
  implements IDotGraphBuilder {

  // //////////////////////////////////////////////////////////
  // 
  // Initialization
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Constructor.
   */
  protected DotGraphBuilder () {
  }

  /**
   * Creates a new instance.
   * @return the new instance.
   */
  public static DotGraphBuilder newDotGraphBuilder () {

    DotGraphBuilder result = new DotGraphBuilder ();

    result.init ();

    return result;
  }

  /**
   * Init method.
   */
  protected void init () {

    BasicGraph graph = BasicGraph.newBasicGraph ();
    HashMap graphAttributes = new HashMap ();
    HashMap nodeAttributes = new HashMap ();
    HashMap arcAttributes = new HashMap ();

    this.setBasicGraph (graph);
    this.setGraphAttributes (graphAttributes);
    this.setDefaultNodeAttributes (nodeAttributes);
    this.setDefaultArcAttributes (arcAttributes);

    // 
    assert this.getBasicGraph () == graph;

    // 
  }

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns the graph.
   * @return the graph.
   */
  protected BasicGraph getBasicGraph () {

    // 
    assert this._basicGraph != null;

    // 
    return this._basicGraph;
  }

  /**
   * The graph to set.
   * @param graph the given graph.
   */
  protected void setBasicGraph (BasicGraph graph) {

    // 
    assert graph != null;

    // 
    this._basicGraph = graph;

    // 
    assert this.getBasicGraph () == graph;

    // 
  }

  /**
   * Returns the arcs.
   * @return the arcs.
   */
  protected HashMap getArcs () {

    // 
    assert this._arcs != null;

    // 
    return this._arcs;
  }

  /**
   * The arcs to set.
   * @param arcs the given arcs.
   */
  protected void setArcs (HashMap arcs) {

    // 
    assert arcs != null;

    // 
    this._arcs = arcs;

    // 
    assert this.getArcs () == arcs;

    // 
  }

  /**
   * Returns the subgraphById.
   * @return the subgraphById.
   */
  protected HashMap getSubgraphById () {

    // 
    assert this._subgraphById != null;

    // 
    return this._subgraphById;
  }

  /**
   * The subgraphById to set.
   * @param subgraphById the given subgraphById.
   */
  protected void setSubgraphById (HashMap subgraphById) {

    // 
    assert subgraphById != null;

    // 
    this._subgraphById = subgraphById;

    // 
    assert this.getSubgraphById () == subgraphById;

    // 
  }

  /**
   * Returns the graphAttributes.
   * @return the graphAttributes.
   */
  protected HashMap getGraphAttributes () {

    // 
    assert this._graphAttributes != null;

    // 
    return this._graphAttributes;
  }

  /**
   * The graphAttributes to set.
   * @param graphAttributes the given graphAttributes.
   */
  protected void setGraphAttributes (HashMap graphAttributes) {

    // 
    assert graphAttributes != null;

    // 
    this._graphAttributes = graphAttributes;

    // 
    assert this.getGraphAttributes () == graphAttributes;

    // 
  }

  /**
   * Returns the defaultNodeAttributes.
   * @return the defaultNodeAttributes.
   */
  protected HashMap getDefaultNodeAttributes () {

    // 
    assert this._defaultNodeAttributes != null;

    // 
    return this._defaultNodeAttributes;
  }

  /**
   * Sets the defaultNodeAttributes.
   * @param defaultNodeAttributes the defaultNodeAttributes to set.
   */
  protected void setDefaultNodeAttributes (HashMap defaultNodeAttributes) {

    // 
    assert defaultNodeAttributes != null;

    // 
    this._defaultNodeAttributes = defaultNodeAttributes;

    // 
    assert this.getDefaultNodeAttributes () == defaultNodeAttributes;

    // 
  }

  /**
   * Returns the defaultArcAttributes.
   * @return the defaultArcAttributes.
   */
  protected HashMap getDefaultArcAttributes () {

    // 
    assert this._defaultArcAttributes != null;

    // 
    return this._defaultArcAttributes;
  }

  /**
   * The defaultArcAttributes to set.
   * @param defaultArcAttributes the given defaultArcAttributes.
   */
  protected void setDefaultArcAttributes (HashMap defaultArcAttributes) {

    // 
    assert defaultArcAttributes != null;

    // 
    this._defaultArcAttributes = defaultArcAttributes;

    // 
    assert this.getDefaultArcAttributes () == defaultArcAttributes;

    // 
  }

  /**
   * Returns the built graph.
   * @return the built graph.
   */
  public BasicGraph getGraph () {

    BasicGraph result = this.getBasicGraph ();
    Set keys = this.getGraphAttributes ().keySet ();

    for (Iterator iter = keys.iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      this.getBasicGraph ().putAttribute (key, this.getGraphAttributes ().get (key));
    }

    // 
    assert result != null;

    // 
    return result;
  }

  // //////////////////////////////////////////////////////////
  // 
  // Basic operations
  // 
  // //////////////////////////////////////////////////////////

  /**
   * @see north.graph.antlr.IDotGraphBuilder#buildSubgraph(java.lang.String, java.util.Map)
   */
  public void buildSubgraph (String id, Map subgraphAttributes) {

    BasicSubgraph subgraph;

    if (!this.getSubgraphById ().containsKey (id)) {
      subgraph = BasicSubgraph.newBasicSubgraph (this.getBasicGraph ());

      this.getSubgraphById ().put (id, subgraph);
      this.getGraph ().addSubgraph (subgraph);
    } else {
      subgraph = (BasicSubgraph) this.getSubgraphById ().get (id);
    }

    Set attributekeys = subgraphAttributes.keySet ();

    for (Iterator iter = attributekeys.iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      subgraph.putAttribute (key, subgraphAttributes.get (key));
    }
  }

  /**
   * @see north.graph.antlr.IDotGraphBuilder#buildArc(java.lang.String, java.lang.String, java.util.HashMap, java.util.List)
   */
  public void buildArc (String tailId, String headId, HashMap attributes, List subgraphList) {

    // 
    assert tailId != null;

    assert headId != null;

    // 
    // The current arc.
    BasicArc arc;

    // Generate the 'dot' id for the arc.
    String arcId = tailId + "->" + headId;

    if (!this.getArcs ().containsKey (arcId)) {

      // Create new nodes if needed.
      this.buildNode (tailId, new HashMap (), subgraphList);
      this.buildNode (headId, new HashMap (), subgraphList);

      // Retreive the corresponding nodes.
      BasicNode tailNode = this.getBasicGraph ().getNode (tailId);
      BasicNode headNode = this.getBasicGraph ().getNode (headId);

      // Add a new arc.
      arc = BasicArc.newBasicArc (tailNode, headNode);

      // Put the arc in the map.
      this.getArcs ().put (arcId, arc);

      // Add the arc in the graph.
      this.getBasicGraph ().addArc (arc);
    } else {
      arc = (BasicArc) this.getArcs ().get (arcId);
    }

    // Adding default attributes
    HashMap defaultAttributes = this.getDefaultArcAttributes ();

    for (Iterator iter = defaultAttributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      arc.putAttribute (key, defaultAttributes.get (key));
    }

    // Adding passed attributes (eventually replacing some default ones)
    for (Iterator iter = attributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      arc.putAttribute (key, attributes.get (key));
    }

    // Adding arc in subgraphs.
    for (Iterator iter = subgraphList.iterator (); iter.hasNext (); ) {
      String subgraphId = (String) iter.next ();

      if (!this.getSubgraphById ().containsKey (subgraphId)) {
        this.buildSubgraph (subgraphId, new HashMap ());
      }

      BasicSubgraph subgraph = (BasicSubgraph) this.getSubgraphById ().get (subgraphId);

      subgraph.addArc (arc);
    }

    // 
    assert this.getBasicGraph ().hasNode (tailId);

    assert this.getBasicGraph ().hasNode (headId);

    // 
  }

  /**
   * @see north.graph.antlr.IDotGraphBuilder#buildNode(java.lang.String, java.util.HashMap, java.util.List)
   */
  public void buildNode (String nodeId, HashMap attributes, List subgraphList) {

    // 
    assert nodeId != null;

    assert attributes != null;

    // Adding the node to the graph if it's not already in it.
    BasicGraph graph = this.getBasicGraph ();

    if (!graph.hasNode (nodeId)) {
      this.getBasicGraph ().addNode (BasicNode.newBasicNode (graph, nodeId, nodeId, this.getClass ().getName ()));
    }

    BasicNode node = this.getBasicGraph ().getNode (nodeId);

    // Adding default attributes
    HashMap defaultAttributes = this.getDefaultNodeAttributes ();

    for (Iterator iter = defaultAttributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      node.putAttribute (key, defaultAttributes.get (key));
    }

    // Adding passed attributes (eventually replacing some default ones)
    for (Iterator iter = attributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      node.putAttribute (key, attributes.get (key));
    }

    // Adding node in subgraphs.
    for (Iterator iter = subgraphList.iterator (); iter.hasNext (); ) {
      String subgraphId = (String) iter.next ();

      if (!this.getSubgraphById ().containsKey (subgraphId)) {
        this.buildSubgraph (subgraphId, new HashMap ());
      }

      BasicSubgraph subgraph = (BasicSubgraph) this.getSubgraphById ().get (subgraphId);

      subgraph.addNode (node);
    }

    // 
    assert this.getBasicGraph ().hasNode (nodeId);

    // 
  }

  /**
   * Adds a graph attribute.
   * @param key the key.
   * @param value the value.
   */
  protected void putGraphAttribute (String key, Object value) {

    // 
    assert key != null;

    assert value != null;

    // 
    HashMap map = this.getGraphAttributes ();

    map.put (key, value);

    // 
    assert map.containsKey (key);

    // 
  }

  /**
   * Adds a default node attribute.
   * @param key the key.
   * @param value the value.
   */
  protected void putDefaultNodeAttribute (String key, Object value) {

    // 
    assert key != null;

    assert value != null;

    // 
    HashMap map = this.getDefaultNodeAttributes ();

    map.put (key, value);

    // 
    assert map.containsKey (key);

    // 
  }

  /**
   * Adds a default arc attribute.
   * @param key the key.
   * @param value the value.
   */
  protected void putDefaultArcAttribute (String key, Object value) {

    // 
    assert key != null;

    assert value != null;

    // 
    HashMap map = this.getDefaultArcAttributes ();

    map.put (key, value);

    // 
    assert map.containsKey (key);

    // 
  }

  /**
   * @see north.graph.antlr.IDotGraphBuilder#putGraphAttributes(java.util.HashMap)
   */
  public void putGraphAttributes (HashMap attributes) {

    // 
    assert attributes != null;

    // 
    // Adding passed attributes (eventually replacing some default ones)
    for (Iterator iter = attributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      this.putGraphAttribute (key, attributes.get (key));
    }
  }

  /**
   * @see north.graph.antlr.IDotGraphBuilder#putDefaultNodeAttributes(java.util.HashMap)
   */
  public void putDefaultNodeAttributes (HashMap attributes) {

    // 
    assert attributes != null;

    // 
    // Adding passed attributes (eventually replacing some default ones)
    for (Iterator iter = attributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      this.putDefaultNodeAttribute (key, attributes.get (key));
    }
  }

  /**
   * @see north.graph.antlr.IDotGraphBuilder#putDefaultArcAttributes(java.util.HashMap)
   */
  public void putDefaultArcAttributes (HashMap attributes) {

    // 
    assert attributes != null;

    // 
    // Adding passed attributes (eventually replacing some default ones)
    for (Iterator iter = attributes.keySet ().iterator (); iter.hasNext (); ) {
      String key = (String) iter.next ();

      this.putDefaultArcAttribute (key, attributes.get (key));
    }
  }

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** The associated graph */
  private BasicGraph _basicGraph;

  /** Built Arcs */
  private HashMap _arcs = new HashMap ();

  /** Subgraphs identifiers */
  private HashMap _subgraphById = new HashMap ();

  /** Graph attributes */
  private HashMap _graphAttributes;

  /** Default node attributes */
  private HashMap _defaultNodeAttributes;

  /** Default arc attributes */
  private HashMap _defaultArcAttributes;
}


/*--- Formatted in North Style ---*/
