/*******************************************************************************
* Copyright (c) 2002-2004, North Bears-SCMBB-ULB
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*    - Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*    - Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*
* Neither the names of North Bears, SCMBB or ULB nor the names of their
* contributors may be used to endorse or promote products derived from
* this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
 *******************************************************************************/

package north.graph.core.builder;



import north.graph.core.IGraph;


/**
 * IMetabolicGraphBuilder class.
 *
 * @author Erick Antezana, Fabian Couche, Didier Croes, Simon De Keyzer, Frederic Fays, Olivier Hubaut and Christian Lemer.
 * @version $Id: IMetabolicGraphBuilder.java,v 1.4 2004/10/13 07:35:10 fabian Exp $
 */
public interface IMetabolicGraphBuilder {

  // //////////////////////////////////////////////////////////
  // 
  // Access
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Returns the built graph.
   * @return the built graph.
   */
  IGraph getGraph ();

  // //////////////////////////////////////////////////////////
  // 
  // Basic operations
  // 
  // //////////////////////////////////////////////////////////

  /**
   * Creates a gene node and adds it to the graph.
   * @param geneId the gene id.
   */
  void buildGene (String geneId);

  /**
   * Creates a polypeptide node and adds it to the graph.
   * @param polypeptideId the polypeptide id.
   */
  void buildPolypeptide (String polypeptideId);

  /**
   * Creates a compound node and adds it to the graph.
   * @param compoundId the compound id.
   */
  void buildCompound (String compoundId);

  /**
   * Creates an expression and adds it to the graph.
   * @param expressionId the expression id.
   * @param geneId the associated gene id.
   * @param polypeptideId the associated polypeptide id.
   */
  void buildExpression (String expressionId, String geneId, String polypeptideId);

  /**
   * Creates a reaction and adds it to the graph.
   * @param reactionId the reaction id.
   * @param substrateIds the associated substrates ids.
   * @param productIds the associated productts ids.
   */
  void buildReaction (String reactionId, String [] substrateIds, String [] productIds);

  /**
   * Creates a catalysis and adds it to the graph.
   * @param catalysisId the catalysis id.
   * @param polypeptideId the associated polypeptide id.
   * @param reactionId the associated rection id.
   */
  void buildCatalysis (String catalysisId, String polypeptideId, String reactionId);

  // //////////////////////////////////////////////////////////
  // 
  // Implementation
  // 
  // //////////////////////////////////////////////////////////

  /** Gene */
  String GENE_TYPE = "Gene";

  /** Polypeptide */
  String POLYPEPTIDE_TYPE = "Polypeptide";

  /** Compound */
  String COMPOUND_TYPE = "Compound";

  /** Expression */
  String EXPRESSION_TYPE = "Expression";

  /** Reaction */
  String REACTION_TYPE = "Reaction";

  /** Catalysis */
  String CATALYSIS_TYPE = "Catalysis";

  /** Substrate */
  String SUBSTRATE_TYPE = "Substrate";

  /** Product */
  String PRODUCT_TYPE = "Product";
}


/*--- Formatted in North Style ---*/
